/*
 * Created on Sep 1, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.curso;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.LinkedList;

import br.unicamp.nied.teleduc.extrator.config.Config;
import br.unicamp.nied.teleduc.extrator.database.ConnectionFactory;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.ferramentas.Anexo;
import br.unicamp.nied.teleduc.extrator.ferramentas.ArquivoTool;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;

/**
 * @author leonel
 *
 */
public abstract class ExtratorFerramenta {
    protected String db;

    protected String dirExtracao;

    protected int[] itensExtrair;

    //    protected Document document;
    //    protected Node xmlNode;
    //    protected String nomeFerramenta;
    protected Curso curso;

    protected Usuario usuario;

    /*
     * Raiz dos arquivos no diretrio de origem
     */
    private String caminhoRaiz;

    public ExtratorFerramenta() {
        super();
    }

    public ExtratorFerramenta(Curso curso, Usuario usr) {
        this(curso);
        this.usuario = usr;
    }

    public ExtratorFerramenta(Curso curso) {
        this();
        this.curso = curso;
        this.caminhoRaiz = null;
        try {
            this.db = curso.getBaseCurso();
            Config config = Config.getConfig();
            this.dirExtracao = (String) config.getProperties().get(
                    "Extract.outputPath");
        } catch (IOException ioex) {
            ioex.printStackTrace();
        }
    }

    /**
     * @deprecated nao precisamos + setar o DB na mao. o mesmo sera setado
     *             automaticamente no construtor utilizando o metodo getCursoDb.
     * @see br.unicamp.nied.teleduc.extrator.curso.ExtratorFerramenta#setDatabase(java.lang.String)
     */
    public void setDatabase(String pDb) {
        this.db = pDb;
    }

    //    /**
    //     * Retorna um XML detalhando a ferramenta
    //     *
    //     * @return
    //     */
    //    public Node getXML() {
    //        return null;
    //    }

    /**
     * Gera um objeto File para cada arquivo pertencente ao item indicado pelo
     * parametro codItem
     *
     * @param codItem
     *            inteiro contendo o codigo do item sendo extraido no momento.
     * @return Collection de File com os arquivos do curso
     * @deprecated Veja ExtratorFerramenta.copiaArquivos(int)
     */
    public Collection getArquivos(int codItem) {
        String caminho = retornaCaminho();
        caminho += "/" + codItem;

        if (hasArquivos(caminho)) {
            File dir = new File(caminho);
            Collection arquivos = retornaArquivosPasta(caminho, dir);
            return arquivos;
        }
        return null;
    }

    /**
     * TODO - otimizao: modificar este mtodo para passar uma Collection como
     * parametro, para no ficar instanciando diversos Collection
     *
     * @param caminho
     * @param dir
     * @return
     */
    private Collection retornaArquivosPasta(String caminho, File dir) {
        Collection arquivos = new LinkedList();
        arquivos = retornaArquivosPastaAux(caminho, dir, arquivos);
        if (arquivos.isEmpty()) {
            return null;
        }
        return arquivos;
    }

    private Collection retornaArquivosPastaAux(String caminho, File dir,
            Collection arquivos) {
        File temp;
        String[] fileNames = dir.list();
        if (fileNames != null) {
            for (int i = 0; i < fileNames.length; i++) {
                temp = new Anexo(caminho + "/" + fileNames[i]);
                arquivos.add(temp);
                if (temp.isDirectory()) {
                    /*
                     * Adicionar recursivamente os arquivos e diretrios deste
                     * subdiretrio
                     */
                    retornaArquivosPastaAux(caminho, temp, arquivos);
                }
            }
        }
        return arquivos;
    }

    /**
     * @deprecated Nao eh mais necessario setar o diretorio na mao. O mesmo eh
     *             um item no arquivo .properties
     * @param pDir
     */
    public void setDirExtracao(String pDir) {
        this.dirExtracao = pDir;
    }

    /**
     * @deprecated nao precisamos + saber os itens.. todos os compartilhados
     *             serao extraidos.
     * @param itens
     */
    public void setItensAExtrair(int itens[]) {
        this.itensExtrair = itens;
    }

    /**
     * Realiza a extracao da ferramenta. Extrair uma ferramenta para HTML
     * consiste em copiar os arquivos dos itens da ferramenta para o diretrio
     * de extrao e gerar arquivos HTML representando a ferramenta. O Extrator
     * deve realizar estas duas funes no mtodo extrair.
     */
    abstract public void extrair() throws ExtractionException;

    /**
     *
     * @param caminho
     * @return
     */
    abstract public String nomeFerramenta();

    /**
     * Verifica se o caminho passado por parametro contem algum arquivo
     *
     * @param caminho =
     *            String contendo o caminho a ser verificado.
     * @return Boolean contendo o resultado da verificacao, caminho contem
     *         arquivos, ou nao?
     */
    public boolean hasArquivos(String caminho) {
        File dir = new File(caminho);
        File arquivos[] = null;
        if (dir.exists()) {
            arquivos = dir.listFiles();

            return (arquivos.length > 0);
        }
        return false;
    }

    /**
     * Consulta o caminho de arquivos na base de dados, e monta o caminho das
     * ferramentas'
     *
     * @param caminho,
     *            recebe essa string que sera concatenada ao caminho da
     *            ferramenta, e util onde o caminho dos arquivos foge do padrao
     *            como em <b>portfolio </b> e <b>exercicios </b>.
     * @return String contendo o caminho dos arquivos da ferramenta
     */
    protected String retornaCaminho(String caminho) {
        String caminhoRaiz = null;
        try {
            caminhoRaiz = retornaCaminhoRaiz();
            if (caminhoRaiz != null) {
                caminhoRaiz += "/" + this.curso.getCodCurso() + "/"
                        + this.nomeFerramenta() + "/" + caminho;
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (DBException e) {
            e.printStackTrace();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return caminhoRaiz;
    }

    /**
     * @see retornaCaminho(String caminho)
     */
    protected String retornaCaminho() {
        String caminho = null;

        try {
            caminho = retornaCaminhoRaiz();
            if (caminho != null) {
                caminho += "/" + this.curso.getCodCurso() + "/"
                        + this.nomeFerramenta();
            }
            //else
            //TODO nao consegui pegar caminho dos arquivos!

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (DBException e) {
            e.printStackTrace();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return caminho;
    }

    /**
     * Devolve o caminho raiz da ferramenta.
     *
     * @return String contendo o caminho raiz
     * @throws IOException
     * @throws FileNotFoundException
     * @throws DBException
     * @throws SQLException
     */
    private String retornaCaminhoRaiz() throws IOException,
            FileNotFoundException, DBException, SQLException {
        if (this.caminhoRaiz == null) {
            String TelEducdb = getTelEducdb();
            ConnectionFactory conFac = ConnectionFactory.getConnectionFactory();
            Connection con = conFac.getConnection(TelEducdb);
            String sql = "select diretorio from Diretorio where item = 'arquivos'";
            PreparedStatement ps = con.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                this.caminhoRaiz = rs.getString("diretorio");
            }
        }
        return this.caminhoRaiz;
    }

    /**
     * consulta o campo extract.baseCurso no arquivo .properties e gera o nome
     * da base do curso que esta sendo extraido no momento.
     *
     * @return String contendo o nome do curso sendo extraido no momento.
     * @throws IOException
     */
    protected String getCursoDb() throws IOException {
        Config config = Config.getConfig();
        String CursoDb = (String) config.getProperties().get(
                "extract.baseCurso");
        return (CursoDb + curso.getCodCurso());
    }

    /**
     * @param curso
     *            The curso to set.
     */
    public void setCurso(Curso curso) {
        this.curso = curso;
    }

    /**
     * Copia todos os arquivos do item indicado pelo parametro codItem para a
     * pasta de arquivos da extracao. Ja monta a arvore de diretorios
     * necessaria, caso a mesma nao exista ou esteja incompleta. Retorna uma
     * Collection de objetos File representando os arquivos na pasta em que
     * foram copiados.
     *
     * @param codItem
     *            o codigo do item sendo extraido no momento.
     * @return TODO
     * @throws ExtractionException
     * @return Collection de objetos File representando os arquivos na pasta em
     *         que foram copiados.
     */
    protected final Collection copiaArquivos(int codItem, String caminho)
            throws ExtractionException {
        File dirDestino = null;
        File dirOrigem = null;
        String strDirOrigem = null;
        String strDirDestino = null;
        try {
            strDirOrigem = retornaCaminho(caminho) + "/" + codItem;
            strDirDestino = montaCaminhoDestino(caminho + "/" + codItem);

            dirOrigem = new File(strDirOrigem);
            dirDestino = new File(strDirDestino);
            return copiaDiretorio(dirOrigem, dirDestino);
        } catch (IOException ioex) {
            throw new ExtractionException("Erro ao copiar arquivos!", ioex);
        }
    }

    /**
     * Copia todos os arquivos do item indicado pelo parametro codItem para a
     * pasta de arquivos da extracao. Ja monta a arvore de diretorios
     * necessaria, caso a mesma nao exista ou esteja incompleta. Retorna uma
     * Collection de objetos File representando os arquivos na pasta de extrao
     * depois da cpia.
     *
     * @param codItem
     *            o codigo do item sendo extraido no momento.
     * @throws ExtractionException
     * @return Collection com objetos File representando cada arquivo na nova
     *         pasta
     */
    protected Collection copiaArquivos(int codItem) throws ExtractionException {
        File dirOrigem = null;
        File dirDestino = null;
        String strDirOrigem = null;
        String strDirDestino = null;
        try {
            strDirOrigem = retornaCaminho() + "/" + codItem;
            strDirDestino = montaCaminhoDestino(codItem);
            dirOrigem = new File(strDirOrigem);
            dirDestino = new File(strDirDestino);
            return copiaDiretorio(dirOrigem, dirDestino);
        } catch (IOException ioex) {
            throw new ExtractionException("Erro ao copiar arquivos!", ioex);
        }
    }

    /**
     * @param file
     * @param file2
     * @throws IOException
     * @throws FileNotFoundException
     */

    protected Collection copiaDiretorio(File dirOrigem, File dirDestino)
            throws FileNotFoundException, IOException {
        Collection c = new LinkedList();
        copiaDiretorioAux(dirOrigem, dirDestino, c, dirDestino
                .getAbsolutePath());
        return c;
    }

    private void copiaDiretorioAux(final File dirOrigem, final File dirDestino,
            final Collection arquivos, final String raizDestino)
            throws FileNotFoundException, IOException {
        /*
         * Andar recursivamente na hierarquia de diretrios, e colocar cada
         * arquivo na collection passada
         */
        final String[] dir = dirOrigem.list();
        if (dir != null) {
            Thread t[] = new Thread[dir.length];

            for (int i = 0; i < dir.length; i++) {

                final String s1 = dir[i];
                Runnable r = new Runnable() {
                    public void run() {
                        final Anexo origem = new Anexo(dirOrigem, s1);
                        final Anexo destino = new Anexo(dirDestino, s1);
                        /*
                         * Marcar o caminho de extrao no arquivo de destino
                         */
                        destino.setCaminhoExtracao(raizDestino);

                        try {
                            if (origem.isDirectory()) {
                                /*
                                 * Copiar recursivamente os arquivos da subpasta
                                 */
                                copiaDiretorioAux(origem, destino, arquivos,
                                        raizDestino);
                            } else {
                                copiaArquivo(origem, destino);
                                arquivos.add(destino);
                            }
                        } catch (FileNotFoundException ex) {
                            ex.printStackTrace();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                };

                t[i] = new Thread(r);
            }

            for (int i = 0; i < t.length; i++) {
                t[i].start();
            }

            for (int i = 0; i < t.length; i++) {
                try {
                    t[i].join();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * @param arquivoOrigem
     * @param arquivoDestino
     * @throws FileNotFoundException
     * @throws IOException
     */
    private void copiaArquivo(final Anexo arquivoOrigem,
            final Anexo arquivoDestino) throws FileNotFoundException,
            IOException {
        /*
         * caso diretorio do arquivo de destino nao exista.. crio os diretorios
         * necessarios
         */
        if (!arquivoDestino.getParentFile().exists()) {
            arquivoDestino.getParentFile().mkdirs();
        }

        //        Thread t = new Thread(
        //            new Runnable() {
        //                public void run() {
        try {
            /*
             * Faz a cpia do arquivo
             */
            FileChannel srcChannel = new FileInputStream(arquivoOrigem)
                    .getChannel();
            FileChannel dstChannel = new FileOutputStream(arquivoDestino)
                    .getChannel();
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());

            /*
             * Verifica se o arquivo de origem tem status especial. Se tiver, o
             * arquivo de destino tambm tem que ter.
             */
            arquivoDestino.setEspecial(arquivoOrigem.isEspecial());

            srcChannel.close();
            dstChannel.close();
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //            }
    //        );
    //        t.start();
    //        try {
    //			t.join();
    //		} catch (InterruptedException e) {
    //			// TODO Auto-generated catch block
    //			e.printStackTrace();
    //		}

    /**
     * Monta o caminho de destino dos arquivos dos itens de uma ferramenta.
     * Geralmente, o caminho de destino dos itens tem a forma
     * <code>arquivos/nomeFerramenta/{$cod_item}</code>. Nestes casos, 
     * melhor usar a verso <code>montaCaminhoDestino(int)</code> que retorna
     * o diretrio com o cdigo do item. Em ferramentas em que todos os arquivos
     * so copiados para um mesmo diretrio, independentemente do item, use esta
     * verso.
     *
     * @return String com o caminho de destino dos itens de uma ferramenta.
     * @throws IOException
     */
    protected String montaCaminhoDestino() throws IOException {
        String caminho = ArquivoTool.montaCaminhoDestino(this.curso,
                this.usuario);
        caminho += "/arquivos/" + nomeFerramenta();
        return caminho;
    }

    /**
     * Monta o caminho para onde os arquivos de determinado item devem ser
     * copiados.
     *
     * @param codItem
     *            Cdigo do item cujos arquivos sero copiados.
     * @return String contendo o caminho completo para onde os arquivos do item
     *         devem ser copiados.
     * @throws IOException
     */
    protected String montaCaminhoDestino(int codItem) throws IOException {
        return this.montaCaminhoDestino() + "/" + codItem;
    }

    /**
     * Monta o caminho para onde os arquivos de determinado item devem ser
     * copiados.
     *
     * @param codItem
     * @return String contendo o caminho completo para onde os arquivos do item
     *         devem ser copiados.
     * @throws IOException
     */
    protected String montaCaminhoDestino(String caminho) throws IOException {
        String caminhoDestino;
        caminhoDestino = montaCaminhoDestino();
        if (caminho != null) {
            caminhoDestino += caminho;
        }
        return caminhoDestino;
    }

    /**
     * Pega o nome da Base do Ambiente no arquivo .properties.
     *
     * @return String contendo o campo extract.BaseAmbiente no arquivo
     *         extrator.properties.
     * @throws IOException
     * @throws FileNotFoundException
     */
    private String getTelEducdb() throws IOException, FileNotFoundException {
        Config config = Config.getConfig();
        String TelEducdb = (String) config.getProperties().get(
                "extract.baseAmbiente");
        return TelEducdb;
    }
}