/*
 * Created on Oct 21, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package br.unicamp.nied.teleduc.extrator.curso;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import br.unicamp.nied.teleduc.extrator.ferramentas.EnderecoWeb;

/**
 * @author bruno
 *
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public abstract class Ferramenta {
    protected int        id;
    protected Date       data;
    protected List       enderecosWeb;
    protected Collection arquivos;

    public Ferramenta() {
        this.enderecosWeb = new ArrayList();
        this.arquivos = new ArrayList();
    }

    /**
     * Gera um objeto File para cada arquivo pertencente ao item indicado ou
     * null se o item no tiver arquivos.
     *
     * @param Curso
     *            Curso ao qual este item de ferramenta pertence
     * @return Collection de File com os arquivos do curso ou null
     * @deprecated use ExtratorFerramenta.copiaArquivo(int)
     */
    public Collection retornaArquivos(Curso curso) {
        String caminho = this.retornaCaminho(curso);
        caminho += "/" + this.id;
        File temp = null;
        if (hasArquivos(caminho)) {
            File dir = new File(caminho);
            Collection arquivos = retornaArquivosPasta(caminho, dir);
            return arquivos;
        } else {
            /*
             * Retornar null se este item no contiver arquivos
             */
            return null;
        }
    }

    /**
     * Gera uma Collection de objetos File para cada arquivo pertencente a este
     * item. O caminho extra  adicionado ao fim do caminho normal e antes do
     * item. Por exemplo, os arquivos da ferramenta Exerccios no se encontram
     * em <code>/home/teleduc/arquivos/{idCurso}/exercicios/</code>, mas em
     * um dos caminhos possveis entre
     * <code>/home/teleduc/arquivos/{idCurso}/exercicios/{questao,enunciado,
     * feedback,alternativa,modelo}/</code>.
     *
     *
     * @param curso
     *            Curso que contm este item.
     * @param caminhoExtra
     *            String extra para colocar no caminho dos arquivos.
     * @return Collection de File com os arquivos do curso
     * @deprecated use ExtratorFerramenta.copiaArquivos(int,String)
     */
    public Collection retornaArquivos(Curso curso, String caminhoExtra) {
        String caminho = retornaCaminho(curso);
        caminho += "/" + caminhoExtra + "/" + this.id;
        File temp = null;
        if (hasArquivos(caminho)) {
            File dir = new File(caminho);
            Collection arquivos = retornaArquivosPasta(caminho, dir);
            return arquivos;
        } else {
            /*
             * Retornar null se este item no contiver arquivos
             */
            return null;
        }
    }

    /**
     * @param caminho
     * @param dir
     * @return
     */
    protected Collection retornaArquivosPasta(String caminho, File dir) {
        Collection c = new ArrayList();
        retornaArquivosPastaAux(caminho, dir, c);
        return c;
    }

    /**
     *
     * @param caminho
     * @param dir
     * @param c
     */
    private void retornaArquivosPastaAux(String caminho, File dir,
            Collection arquivos) {
        File temp;
        String[] fileNames = dir.list();
        if (fileNames.length > 0) {
            for (int i = 0; i < fileNames.length; i++) {
                temp = new File(caminho + "/" + fileNames[i]);
                arquivos.add(temp);
                if (temp.isDirectory()) {
                    retornaArquivosPastaAux((caminho + "/" + fileNames[i]),
                            temp, arquivos);
                }
            }
        }
    }

    protected String retornaCaminho(Curso curso) {
        String caminho = curso.getCaminhoArquivos() + "/" + nomeFerramenta();
        return caminho;
    }

    /**
     * Verifica se o caminho passado por parametro contem algum arquivo
     *
     * @param caminho =
     *            String contendo o caminho a ser verificado.
     * @return boolean contendo o resultado da verificacao, caminho contem
     *         arquivos, ou nao
     * @deprecated use ExtratorFerramenta.copiaArquivos(int) e
     *             Ferramenta.getArquivos()
     */
    public boolean hasArquivos(String caminho) {
        File dir = new File(caminho);
        File arquivos[] = null;
        if (dir.exists()) {
            arquivos = dir.listFiles();
            return (arquivos.length > 0);
        }
        return false;
    }

    /**
     * @return Returns the data.
     */
    public Date getData() {
        return data;
    }

    /**
     * Data em formato UnixTime. (Nmero de <b>segundos </b> desde 01/Jan/1970)
     *
     * @return
     */
    public long getDataUT() {
        return this.data.getTime() / 1000L;
    }

    /**
     * Indica o diretrio onde se deve buscar os arquivos desta ferramenta. Deve
     * ser sobreescrito pelas classes filhas. Por exemplo, para a ferramenta
     * Agenda, os arquivos se encontram em
     * <code>/home/teleduc/arquivos/{idCurso}/agenda/</code>, e este mtodo
     * retorna a String <code>"agenda"</code>.
     *
     * @return String representando o nome da ferramenta no sistema de arquivos
     *         do TelEduc
     */
    abstract protected String nomeFerramenta();

    /**
     * Configura a data em java.util.Date, a partir do nmero de <b>segundos
     * </b> do UnixTime Assim: <code>
     * ...
     * java.sql.ResultSet rs = stmt.executeQuery("select data from Tabela");
     * long segundos = rs.getLong("data");
     * item.setDate(segundos);
     * ...
     * </code>
     *
     * @param data
     *            Data em <b>segundos </b> do UnixTime
     */
    public void setData(long l) {
        this.data = new Date(l * 1000L);
    }

    public void setData(Date d) {
        this.data = d;
    }

    /**
     * Retorna uma Collection representando os arquivos do item. Se o item no
     * tiver arquivos, retorna null
     *
     * @return Collection com os arquivos do curso, ou null
     */
    public Collection getArquivos() {
        return arquivos;
    }

    public boolean temEnderecos() {
        return !this.enderecosWeb.isEmpty();
    }

    public boolean temArquivos() {
        return !this.arquivos.isEmpty();
    }

    public void setArquivos(Collection arquivos) {
        this.arquivos = arquivos;
    }

    public void addArquivo(File file) {
        this.arquivos.add(file);
    }

    /**
     *
     * @return Os endereos da Web associados a este item
     */
    public List getEnderecosWeb() {
        return enderecosWeb;
    }

    /**
     *
     * @param enderecos
     */
    public void setEnderecosWeb(List enderecos) {
        this.enderecosWeb = enderecos;
    }

    /**
     *
     * @param end
     */
    public void addEnderecoWeb(EnderecoWeb end) {
        this.enderecosWeb.add(end);
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    // ------------------------------------------------------------------------

    public int hashCode() {
        return this.id;
    }

    public boolean equals(Object obj) {
        if (obj instanceof Ferramenta) {
            Ferramenta f = (Ferramenta) obj;
            return f.id == this.id;
        }
        return false;
    }
}