/*
 * Created on Jan 31, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas;

/**
 * @author leonel
 *
 * Esta classe realiza transformaes sobre Strings para torn-las mais
 * apresentveis em HTML, como transformar quebras de linha em BR, espaos em
 * nbsp.
 *
 */
public class HTMLTool {
    /**
     *
     */
    public HTMLTool() {
        super();
    }

    /**
     * Substitui cada quebra de linha da String passada por BR
     *
     * @param s
     *            String na qual efetuar a substituio
     * @return A nova String, com BR no lugar de quebra de linha
     */
    public String enter2br(String s) {
        return s.replaceAll("\r", "").replaceAll("\n", "<br />\n").trim();
    }

    /**
     * Substitui &lt; e &gt; por &amp;lt; e &amp;gt;, de modo a escrever tags.
     *
     * @param s
     *            String na qual remover as tags
     * @return Nova String, com tags removidas
     */
    public String limpaTags(String s) {
        return s.replaceAll("<", "&lt;").replaceAll(">", "&gt;").trim();
    }

    /**
     * Aplica os outros mtodos  String passada.
     *
     * @param s
     *            String a limpar
     * @return Nova String, limpa
     */
    public String clean(String s) {
        if (s == null) {
            throw new NullPointerException("Parmetro no pode ser null");
        }
        s = escapeHTML(s);
        s = enter2br(s);
        return s;
    }

    /**
     * Escapa caracteres especiais para a notao HTML.
     * Inspirado em
     * http://www.rgagnon.com/javadetails/java-0306.html.
     * Inspirado tambm pelo plugin AnyEdit para Eclipse:
     * http://andrei.gmxhome.de/anyedit/links.html
     *
     * @param s
     *            String da qual escapar os caracteres
     * @return String com caracteres escapados
     */
    public String escapeHTML(String s) {
        StringBuffer sb = new StringBuffer();
        int n = s.length();
        for (int i = 0; i < n; i++) {
            char c = s.charAt(i);
            switch (c) {
                case '<' :        sb.append("&lt;"        );   break;
                case '>' :        sb.append("&gt;"        );   break;
                case '&' :        sb.append("&amp;"       );   break;
                case '"' :        sb.append("&quot;"      );   break;
                case '' :        sb.append("&agrave;"    );   break;
                case '' :        sb.append("&aacute;"    );   break;
                case '' :        sb.append("&Agrave;"    );   break;
                case '' :        sb.append("&Aacute;"    );   break;
                case '' :        sb.append("&acirc;"     );   break;
                case '' :        sb.append("&Acirc;"     );   break;
                case '' :        sb.append("&auml;"      );   break;
                case '' :        sb.append("&Auml;"      );   break;
                case '' :        sb.append("&aring;"     );   break;
                case '' :        sb.append("&Aring;"     );   break;
                case '' :        sb.append("&atilde;"    );   break;
                case '' :        sb.append("&Atilde;"    );   break;
                case '' :        sb.append("&aelig;"     );   break;
                case '' :        sb.append("&AElig;"     );   break;
                case '' :        sb.append("&ccedil;"    );   break;
                case '' :        sb.append("&Ccedil;"    );   break;
                case '' :        sb.append("&eacute;"    );   break;
                case '' :        sb.append("&Eacute;"    );   break;
                case '' :        sb.append("&egrave;"    );   break;
                case '' :        sb.append("&Egrave;"    );   break;
                case '' :        sb.append("&ecirc;"     );   break;
                case '' :        sb.append("&Ecirc;"     );   break;
                case '' :        sb.append("&euml;"      );   break;
                case '' :        sb.append("&Euml;"      );   break;
                case '' :        sb.append("&iacute;"    );   break;
                case '' :        sb.append("&Iacute;"    );   break;
                case '' :        sb.append("&iuml;"      );   break;
                case '' :        sb.append("&Iuml;"      );   break;
                case '' :        sb.append("&otilde;"    );   break;
                case '' :        sb.append("&Otilde;"    );   break;
                case '' :        sb.append("&oacute;"    );   break;
                case '' :        sb.append("&ocirc;"     );   break;
                case '' :        sb.append("&Oacute;"    );   break;
                case '' :        sb.append("&Ocirc;"     );   break;
                case '' :        sb.append("&ouml;"      );   break;
                case '' :        sb.append("&Ouml;"      );   break;
                case '' :        sb.append("&oslash;"    );   break;
                case '' :        sb.append("&Oslash;"    );   break;
                case '' :        sb.append("&szlig;"     );   break;
                case '' :        sb.append("&ugrave;"    );   break;
                case '' :        sb.append("&Ugrave;"    );   break;
                case '' :        sb.append("&uacute;"    );   break;
                case '' :        sb.append("&Uacute;"    );   break;
                case '' :        sb.append("&ucirc;"     );   break;
                case '' :        sb.append("&Ucirc;"     );   break;
                case '' :        sb.append("&uuml;"      );   break;
                case '' :        sb.append("&Uuml;"      );   break;
                case '\u00AE' :   sb.append("&reg;"       );   break;
                case '\u00A9' :   sb.append("&copy;"      );   break;
                case '\u20A0' :   sb.append("&euro;"      );   break;
                case '\u20AC' :   sb.append("&#8364;"     );   break;
                // be carefull with this one (non-breaking whitee space)
                case ' ':         sb.append(' '); break;
                default :  {
                    //Submitted by S. Bayer.
                    /*
                     * O caracter no  nenhum dos conhecidos, verifique se 
                     * Unicode. Os caracteres Unicode so os que esto antes do
                     * espao (&nbsp).
                     *
                     */
                    int ci = 0xffff & c;
                    if (ci < 160) {
                        // caracter comum.
                        sb.append(c);
                    } else {
                        // caracter estranho, utilizamos a sintaxe Unicode.
                        sb.append("&#" + ci + ";");
                    }
                }
            }
        }
        return sb.toString();
    }
}
