/*
 * Created on Feb 25, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.acessos;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.database.ConnectionFactory;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.ferramentas.CalendarTool;
import br.unicamp.nied.teleduc.extrator.ferramentas.FerramentaTool;
import br.unicamp.nied.teleduc.extrator.ferramentas.grupos.Grupo;
import br.unicamp.nied.teleduc.extrator.ferramentas.grupos.GrupoFactory;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;

/**
 * @author leonel
 *
 * Responsvel por retornar as quantidades de acessos a determinada ferramenta
 * no curso, em determinada data, por determinado usurio.
 */
public class AcessosFerramenta {
    // Map<Usuario, Map<Date, Integer>>
    private Map acessosUsuarios = null;

    // Map<Grupo, Map<Date, Integer>>
    private Map acessosGrupos = null;

    private Curso curso;

    private byte ferramenta;

    private CalendarTool calendar;

    private Date dataInicio;

    private Date dataFim;

    /*
     * Vamos utilizar inicializacao tardia. Este booleano indica se o objeto j
     * pesquisou a base de dados para pegar os acessos de cada usurio no dia
     */
    private boolean inicializadoGrupos = false;

    private boolean inicializadoUsuario = false;

    public AcessosFerramenta(Curso curso, byte ferramenta, Date data)
            throws ExtractionException {
        this(curso, ferramenta, data, data);
    }

    /**
     * @throws ExtractionException
     *
     */
    public AcessosFerramenta(Curso curso, byte ferramenta, Date dataInicio,
            Date dataFim) throws ExtractionException {
        super();
        this.curso = curso;
        this.ferramenta = ferramenta;
        switch (ferramenta) {
        case FerramentaTool.ENTRADA_NO_AMBIENTE: /* falls through */
        case FerramentaTool.ESTRUTURA_DO_AMBIENTE: /* falls through */
        case FerramentaTool.DINAMICA: /* falls through */
        case FerramentaTool.AGENDA: /* falls through */
        case FerramentaTool.AVALIACOES: /* falls through */
        case FerramentaTool.ATIVIDADES: /* falls through */
        case FerramentaTool.MATERIAL: /* falls through */
        case FerramentaTool.LEITURAS: /* falls through */
        case FerramentaTool.PERGUNTAS_FREQUENTES: /* falls through */
        case FerramentaTool.EXERCICIOS: /* falls through */
        case FerramentaTool.PARADA_OBRIGATORIA: /* falls through */
        case FerramentaTool.MURAL: /* falls through */
        case FerramentaTool.FORUNS: /* falls through */
        case FerramentaTool.BATEPAPO: /* falls through */
        case FerramentaTool.CORREIO: /* falls through */
        case FerramentaTool.GRUPOS: /* falls through */
        case FerramentaTool.PERFIL: /* falls through */
        case FerramentaTool.DIARIO: /* falls through */
        case FerramentaTool.PORTFOLIO: /* falls through */
        case FerramentaTool.ACESSOS: /* falls through */
        case FerramentaTool.INTERMAP: /* falls through */
            break;
        /*
         * Se o cdigo da ferramenta no for nenhum dos esperados, lanamos uma
         * exceo
         */
        default:
            throw new IllegalArgumentException("Cdigo de ferramenta Invlido");
        }

        this.acessosUsuarios = new HashMap();
        this.acessosGrupos = new HashMap();

        /*
         * Canonizamos as datas antes de buscar os acessos no BD
         */
        this.calendar = new CalendarTool();
        this.dataInicio = calendar.zeroHora(dataInicio);
        this.dataFim = calendar.hora23(dataFim);

        // this.buscaQuantidadeAcessosUsuarios(dataInicio, dataFim);
        // this.buscaQuantidadeAcessosGrupos(dataInicio, dataFim);
    }

    /**
     * Retorna os acessos a uma determinada ferramenta entre duas datas
     *
     * @param d1
     *            Date data de incio
     * @param d2
     *            Date data final
     * @return
     * @throws ExtractionException
     */
    public List getAcessos(Date d1, Date d2) throws ExtractionException {
        CalendarTool calendarTool = new CalendarTool();
        d1 = calendarTool.zeroHora(d1);
        d2 = calendarTool.hora23(d2);

        ResultSet rs = null;
        PreparedStatement stmt = null;

        try {
            String f = null;
            if (this.ferramenta == FerramentaTool.ENTRADA_NO_AMBIENTE) {
                f = "cod_ferramenta is null";
            } else {
                f = "cod_ferramenta = " + this.ferramenta;
            }

            Connection conn = ConnectionFactory.getConnectionFactory()
                    .getConnection(this.curso.getBaseCurso());
            String sql = "select cod_usuario as 'idUsuario', data,"
                    + " cod_ferramenta as 'idFerramenta'"
                    + " from Curso_acessos" + " where cod_usuario >= 0"
                    + " and cod_ferramenta is null" + " and " + d1.getTime()
                    / 1000L + " <= data" + " and data <= " + d2.getTime()
                    / 1000L;
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();

            UsuarioFactory uf = UsuarioFactory.getInstance(this.curso);

            List l = new ArrayList(ConnectionFactory.numeroDeLinhas(rs));
            while (rs.next()) {
                Date data = new Date(rs.getLong("data") * 1000L);
                byte ferramenta = rs.getByte("idFerramenta");
                Usuario usr = uf.getUsuario(rs.getInt("idUsuario"));

                Acesso a = new Acesso(usr, data, ferramenta);
                l.add(a);
            }
            return l;
        } catch (SQLException sqlex) {
            throw new ExtractionException("Erro ao extrair acessos no perdo "
                    + d1 + " a " + d2, sqlex);
        } catch (DBException e) {
            throw new ExtractionException("Erro ao extrair acessos no perdo "
                    + d1 + " a " + d2, e);
        } catch (IOException e) {
            throw new ExtractionException("Erro ao extrair acessos no perdo "
                    + d1 + " a " + d2, e);
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException sqlex) {
                    // ignore
                }
            }
            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException sqlex) {
                    // ignore
                }
            }
        }
    }

    /**
     * Retorna todos os acessos a uma ferramenta em determinada data.
     *
     * @param d
     *            Date data da qual retornar os acessos.
     * @return
     * @throws ExtractionException
     */
    public List getAcessos(Date d) throws ExtractionException {
        CalendarTool calendar = new CalendarTool();
        Date d1 = calendar.zeroHora(d);
        Date d2 = calendar.hora23(d);

        return this.getAcessos(d1, d2);
    }

    /**
     * Retorna a quantidade de acesso do grupo a esta ferramenta no dia da data
     * d.
     *
     * @param usuario
     *            O usurio do qual queremos calcular os acesso
     * @param byte
     *            Ferramenta cdigo da ferramenta que o usurio acesso
     * @param d
     *            dia da quantidade de acessos a calcular.
     * @return
     * @throws ExtractionException
     */
    public int quantidadeAcessos(Grupo grupo, Date d)
            throws ExtractionException {
        if (!this.inicializadoGrupos) {
            this.buscaQuantidadeAcessosGrupos(this.dataInicio, this.dataFim);
            this.inicializadoGrupos = true;
        }
        return this.getQuantidadeAcessos(grupo, d);
    }

    /**
     * Retorna a quantidade de acessos de um usurio a esta ferramenta no dia da
     * data d.
     *
     * @param usuario
     *            O usurio do qual queremos calcular os acesso
     * @param byte
     *            Ferramenta cdigo da ferramenta que o usurio acesso
     * @param d
     *            dia da quantidade de acessos a calcular.
     * @return
     * @throws ExtractionException
     */
    public int quantidadeAcessos(Usuario usuario, Date d)
            throws ExtractionException {
        if (!this.inicializadoUsuario) {
            this.buscaQuantidadeAcessosUsuarios(this.dataInicio, this.dataFim);
            this.inicializadoUsuario = true;
        }
        return this.getQuantidadeAcessos(usuario, d);
    }

    /**
     * Retorna a soma das quantidades de acessos de cada usurio do grupo
     *
     * @throws ExtractionException
     *
     */
    public int totalAcessos(Grupo g) throws ExtractionException {
        if (!this.inicializadoGrupos) {
            this.buscaQuantidadeAcessosGrupos(this.dataInicio, this.dataFim);
            this.inicializadoGrupos = true;
        }
        int totalAcessos = 0;
        Map m = (Map) this.acessosGrupos.get(g);
        if (m != null) {
            for (Iterator itr = m.values().iterator(); itr.hasNext();) {
                Integer nAcessos = (Integer) itr.next();
                totalAcessos += nAcessos.intValue();
            }
        } else {
            /*
             * Grupo no existe no BD. Ento deve ser um grupo artificial criado
             * pelo Extrator, para agrupar somente alunos ou somente Formadores.
             * Calculamos a soma de acessos de cada um
             */
            for (Iterator itr = g.getUsuarios().iterator(); itr.hasNext();) {
                Usuario usr = (Usuario) itr.next();
                totalAcessos += this.totalAcessos(usr);
            }
        }
        return totalAcessos;
    }

    /**
     * Retorna a quantidade total de acessos do usurio no perodo especificado
     *
     * @throws ExtractionException
     *
     */
    public int totalAcessos(Usuario usuario) throws ExtractionException {
        if (!this.inicializadoUsuario) {
            this.buscaQuantidadeAcessosUsuarios(this.dataInicio, this.dataFim);
            this.inicializadoUsuario = true;
        }
        int totalAcessos = 0;
        Map m = (Map) this.acessosUsuarios.get(usuario);
        if (m != null) {
            for (Iterator itr = m.values().iterator(); itr.hasNext();) {
                Integer nAcessos = (Integer) itr.next();
                if (nAcessos != null) {
                    totalAcessos += nAcessos.intValue();
                }
            }
        }
        return totalAcessos;
    }

    /**
     * Retorna a soma das quantidades de acessos dos usurios no dia
     * especificado
     *
     * @throws ExtractionException
     *
     */
    public int totalAcessos(Date d) throws ExtractionException {
        if (!this.inicializadoUsuario) {
            this.buscaQuantidadeAcessosUsuarios(this.dataInicio, this.dataFim);
            this.inicializadoUsuario = true;
        }
        int totalAcessos = 0;
        for (Iterator itr = this.acessosUsuarios.values().iterator(); itr
                .hasNext();) {
            Map m = (Map) itr.next();
            if (m != null) {
                Integer nAcessos = (Integer) m.get(d);
                if (nAcessos != null) {
                    totalAcessos += nAcessos.intValue();
                }
            }
        }
        return totalAcessos;
    }

    /**
     * Soma a quantidade total de acessos para todos os usurios no perodo
     *
     * @return
     * @throws ExtractionException
     */
    public int totalAcessos() throws ExtractionException {
        // Map<Usuario, Map<Date, Integer>>
        // private Map acessosUsuarios = null;
        /*
         * Para cada usuario, Para cada data em que este usurio acessou Some o
         * nmero de acessos na data
         */
        int totalAcessos = 0;
        for (Iterator i1 = acessosUsuarios.values().iterator(); i1.hasNext();) {
            Map m = (Map) i1.next();
            for (Iterator i2 = m.values().iterator(); i2.hasNext();) {
                Integer nAcessos = (Integer) i2.next();
                totalAcessos += nAcessos.intValue();
            }
        }
        return totalAcessos;
    }

    private void buscaQuantidadeAcessosUsuarios(Date d1, Date d2)
            throws ExtractionException {
        ResultSet rs = null;
        PreparedStatement stmt = null;
        try {
            String sqlFerramenta = null;
            if (this.ferramenta == FerramentaTool.ENTRADA_NO_AMBIENTE) {
                sqlFerramenta = " cod_ferramenta is null ";
            } else {
                sqlFerramenta = " cod_ferramenta=" + this.ferramenta;
            }

            String sql = "select cod_usuario, count(*) as 'qtdeAcessos', FROM_UNIXTIME(data, '%d-%m-%Y') as 'dataST'"
                    + " from Curso_acessos where "
                    + sqlFerramenta
                    + " and "
                    + d1.getTime()
                    / 1000L
                    + " <= data"
                    + " and data <= "
                    + d2.getTime()
                    / 1000L
                    + " and cod_usuario >= 0"
                    + " group by cod_usuario, dataST";

            UsuarioFactory uf = UsuarioFactory.getInstance(this.curso);
            DateFormat df = new SimpleDateFormat("dd-MM-yyyy");

            Connection conn = ConnectionFactory.getConnectionFactory()
                    .getConnection(this.curso.getBaseCurso());
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();
            while (rs.next()) {
                Usuario usr = uf.getUsuario(rs.getInt("cod_usuario"));
                Date d;
                try {
                    d = df.parse(rs.getString("dataST"));
                } catch (ParseException e) {
                    e.printStackTrace();
                    continue;
                }
                int nAcessos = rs.getInt("qtdeAcessos");
                this.setQuantidadeAcessos(usr, d, nAcessos);
            }
        } catch (SQLException ex) {
            throw new ExtractionException("Erro ao extrair acessos", ex);
        } catch (DBException ex) {
            throw new ExtractionException("Erro ao extrair acessos", ex);
        } catch (IOException ex) {
            throw new ExtractionException("Erro ao extrair acessos", ex);
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException sqlex) {
                    // ignore
                }
            }
            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException sqlex) {
                    // ignore
                }
            }
        }
    }

    private void buscaQuantidadeAcessosGrupos(Date d1, Date d2)
            throws ExtractionException {
        ResultSet rs = null;
        PreparedStatement stmt = null;
        try {
            String sqlFerramenta = null;
            if (this.ferramenta == FerramentaTool.ENTRADA_NO_AMBIENTE) {
                sqlFerramenta = " cod_ferramenta is null ";
            } else {
                sqlFerramenta = " cod_ferramenta=" + this.ferramenta;
            }

            String sql = "select GU.cod_grupo as cod_grupo, count(*) as 'qtdeAcessos', FROM_UNIXTIME(data, '%d-%m-%Y') as 'dataST'"
                    + " from Grupos as G"
                    + " left join Grupos_usuario as GU on G.cod_grupo=GU.cod_grupo"
                    + " left join Curso_acessos as CA on GU.cod_usuario=CA.cod_usuario"
                    + " where "
                    + sqlFerramenta
                    + "   and G.status != 'X'"
                    + "   and "
                    + d1.getTime()
                    / 1000L
                    + " <= CA.data"
                    + "   and CA.data <= "
                    + d2.getTime()
                    / 1000L
                    + " group by cod_grupo, dataST"
                    + " order by cod_grupo, dataST";

            Connection conn = ConnectionFactory.getConnectionFactory()
                    .getConnection(this.curso.getBaseCurso());
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();

            GrupoFactory gf = GrupoFactory.getInstance(this.curso);
            DateFormat df = new SimpleDateFormat("dd-MM-yyyy");

            while (rs.next()) {
                Grupo g = gf.getGrupo(rs.getInt("cod_grupo"));
                Date d;
                try {
                    d = df.parse(rs.getString("dataST"));
                } catch (ParseException ex) {
                    ex.printStackTrace();
                    continue;
                }
                int nAcessos = rs.getInt("qtdeAcessos");
                this.setQuantidadeAcessos(g, d, nAcessos);
            }
        } catch (SQLException ex) {
            throw new ExtractionException("Erro ao extrair acessos", ex);
        } catch (DBException ex) {
            throw new ExtractionException("Erro ao extrair acessos", ex);
        } catch (IOException ex) {
            throw new ExtractionException("Erro ao extrair acessos", ex);
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException sqlex) {
                    // ignore
                }
            }
            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException sqlex) {
                    // ignore
                }
            }
        }
    }

    /**
     *
     * @param usuario
     * @param d
     * @param nAcessos
     */
    private void setQuantidadeAcessos(Usuario usuario, Date d, int nAcessos) {
        /*
         * Canoniza a data para as zero-hora do dia.
         */
        d = this.calendar.zeroHora(d);
        /*
         * Associar o nmero de acessos  data e ao usurio
         */
        Map m = (Map) this.acessosUsuarios.get(usuario);
        if (m == null) {
            m = new HashMap();
            this.acessosUsuarios.put(usuario, m);
        }
        m.put(d, new Integer(nAcessos));
    }

    private void setQuantidadeAcessos(Grupo grupo, Date d, int nAcessos) {
        /*
         * Canoniza a data para as zero-hora do dia.
         */
        d = this.calendar.zeroHora(d);
        /*
         * Associar o nmero de acessos  data e ao grupo
         */
        Map m = (Map) this.acessosGrupos.get(grupo);
        if (m == null) {
            m = new HashMap();
            this.acessosGrupos.put(grupo, m);
        }
        m.put(d, new Integer(nAcessos));
    }

    private int getQuantidadeAcessos(Usuario usuario, Date d) {
        Map m = (Map) this.acessosUsuarios.get(usuario);
        if (m == null) {
            return 0;
        }
        Integer nAcessos = (Integer) m.get(d);
        if (nAcessos == null) {
            return 0;
        }
        return nAcessos.intValue();
    }

    private int getQuantidadeAcessos(Grupo grupo, Date d) {
        Map m = (Map) this.acessosGrupos.get(grupo);
        if (m == null) {
            return 0;
        }
        Integer nAcessos = (Integer) m.get(d);
        if (nAcessos == null) {
            return 0;
        }
        return nAcessos.intValue();
    }
}