/*
 * Created on Feb 24, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.acessos;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.TreeMap;

import br.unicamp.nied.teleduc.extrator.ferramentas.CalendarTool;

/**
 * @author leonel
 * 
 * Esta classe prov um Iterator que itera sobre todos os dias entre dois dias
 * definidos.
 */
public class Calendario {
    private Date dataInicio;

    private Date dataFim;

    /**
     * Constroi um calendrio com uma data de incio e uma data de fim. O
     * iterator vai retornar todos os dias entre estas datas.
     */
    public Calendario(Date dataInicio, Date dataFim) {
        this.dataInicio = dataInicio;
        this.dataFim = dataFim;
    }

    /**
     * Retorna uma mapa <Date, Integer> que mapeia um Date representando um ms
     * no nmero de dias que dura aquele ms.
     * 
     * Retirado de
     * http://javaalmanac.com/egs/java.util/GetDaysInMonth.html?l=rel
     */
    public TreeMap meses() {
        TreeMap map = new TreeMap();
        GregorianCalendar calendar = new GregorianCalendar();
        CalendarTool tool = new CalendarTool();

        calendar.setTime(dataFim);
        calendar.add(Calendar.HOUR_OF_DAY, 12);

        Date dFim = calendar.getTime();

        int m2 = calendar.get(Calendar.MONTH);
        calendar.setTime(dataInicio);
        int m1 = calendar.get(Calendar.MONTH);
        int nMeses = m2 - m1 + 1;
        if (nMeses < 0) {
            nMeses += 12;
        }

        while (nMeses-- > 0) {
            int m = calendar.get(Calendar.MONTH);
            int dias = 0;
            Date d = calendar.getTime();

            while (calendar.get(Calendar.MONTH) == m
                    && calendar.getTime().compareTo(dFim) < 0) {
                dias++;

                int dia1 = calendar.get(Calendar.DAY_OF_MONTH);
                calendar.add(Calendar.DAY_OF_MONTH, 1);
                int dia2 = calendar.get(Calendar.DAY_OF_MONTH);
                if (dia1 == dia2) {
                    // Horrio de vero
                    // d1 est s 00:00 hora e d2 est s 23:00 do mesmo dia.
                    // somar mais um dia e marcar s zero hora
                    calendar.add(Calendar.DAY_OF_MONTH, 1);
                    calendar.setTime(tool.zeroHora(calendar.getTime()));
                }
            }
            map.put(d, new Integer(dias));
        }

        return map;
    }

    public List toList() {
        List l = new ArrayList();
        for (DateIterator itr = this.dateIterator(); itr.hasNext();) {
            Date d = itr.next();
            l.add(d);
        }
        return l;
    }

    private DateIterator dateIterator() {
        return new DateIterator(this.dataInicio, this.dataFim);
    }

    /**
     * Classe interna que auxilia na criao de uma lista.  private e NO
     * implementa java.util.Iterator de propsito. Ao invs de utilizar esta,
     * utilize:
     * <code>List l = calendario.toList(); Iterator i = l.iterator();</code>
     */
    private class DateIterator {
        private GregorianCalendar calendar;

        private CalendarTool tool;

        private Date dataFim;

        private DateIterator(Date dataInicio, Date dataFim) {
            // Comear s 00:00:00 do dia da data
            this.tool = new CalendarTool();

            this.calendar = new GregorianCalendar();
            this.calendar.setTime(dataInicio);
            tool.setZeroHora(calendar);

            this.dataFim = tool.hora23(dataFim);
        }

        /**
         * Retorna um date representando um dia entre a data de incio e a de
         * fim. A data est sempre  zero-hora.
         */
        private Date next() {
            int dia1 = this.calendar.get(Calendar.DAY_OF_MONTH);
            Date d = this.calendar.getTime();
            this.calendar.add(Calendar.DAY_OF_MONTH, 1);

            int dia2 = this.calendar.get(Calendar.DAY_OF_MONTH);
            if (dia1 == dia2) {
                /*
                 * Cdigo de correo para horrio de vero. Se dia1  o dia do
                 * horrio de vero s 00:00, ento dia1 + 1 == mesmo dia, 23:00
                 * Na volta do horrio de vero para o horrio normal, parece
                 * no haver problema.
                 */
                this.calendar.add(Calendar.DAY_OF_MONTH, 1);
            }
            tool.setZeroHora(this.calendar);
            return d;
        }

        private boolean hasNext() {
            /*
             * S h mais dias a iterar se o calendar de iterao estiver
             * anterior  data final.
             */
            return this.calendar.getTime().compareTo(this.dataFim) < 0;
        }
    }
}