/*
 * Created on Feb 22, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.acessos;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.tools.generic.DateTool;
import org.apache.velocity.tools.generic.MathTool;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.ferramentas.CalendarTool;
import br.unicamp.nied.teleduc.extrator.ferramentas.FerramentaTool;
import br.unicamp.nied.teleduc.extrator.ferramentas.grupos.Grupo;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;
import br.unicamp.nied.teleduc.transformador.Transform;

/**
 * @author leonel
 *
 */
public class TransformAcessos extends Transform {
    private Comparator comparatorUsuario = new Comparator() {
        public int compare(Object obj1, Object obj2) {
            Usuario usr1 = (Usuario) obj1;
            Usuario usr2 = (Usuario) obj2;
            return usr1.getNome().compareToIgnoreCase(usr2.getNome());
        }
    };


    /**
     * @param curso
     * @param usuario
     * @param nomeFerramenta
     * @throws IOException
     * @throws ExtractionException
     */
    public TransformAcessos(Curso curso, Usuario usuario) throws IOException,
            ExtractionException {
        super(curso, usuario, "acessos");
    }

    public void geraArquivosHTML() throws Exception {
        VelocityEngine ve = super.createVelocityEngine();
        VelocityContext context = super.createVelocityContext();

        /*
         * 1. Gerar pgina de entrada
         */
        geraEntrada(ve, context);

        /*
         * 2. Gerar relatrio de Acessos
         */
        geraRelatorioAcessos(ve, context);

        /*
         * 3. Gerar relatrio de Frequencia
         */
        geraRelatorioFrequencia(ve);

        /*
         * 4. Gerar os css's
         */
        geraEstilos(ve);

      }

    /**
     * @param ve
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraEstilos(VelocityEngine ve)
            throws ResourceNotFoundException, ParseErrorException, Exception {
        // TODO Auto-generated method stub

        Template t;
        Writer writer;
        VelocityContext context = createVelocityContext();

        t = ve.getTemplate("templates/acessos/estiloFrequenciaPeriodo.vsl");
        writer = createWriter(this.dirExtracao + "/estiloFrequenciaPeriodo.css");
        mergeTemplate(t, context, writer);

        t = ve.getTemplate("templates/acessos/estiloDiario.vsl");
        writer = createWriter(this.dirExtracao + "/estiloDiario.css");
        mergeTemplate(t, context, writer);

        t = ve.getTemplate("templates/acessos/estiloPeriodoParticipante.vsl");
        writer = createWriter(this.dirExtracao
                + "/estiloPeriodoParticipante.css");
        mergeTemplate(t, context, writer);
    }

    /**
     * @param ve
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraRelatorioFrequencia(final VelocityEngine ve)
            throws ResourceNotFoundException, ParseErrorException, Exception {
        /*
         * Dividimos a durao do curso em subperodos de 2 semanas.
         *
         * Passamos no contexto um map <Date,Date>, onde o Key e o Value so
         * dois Date que indicam o incio e o fim de um subperodo.
         */
        AcessosCurso acessos = new AcessosCurso(curso);
        final Date dataInicio = acessos.getPrimeiroAcesso();
        final Date dataFim = acessos.getUltimoAcesso();
        final Map periodos = divideSubPeriodos(dataInicio, dataFim, 21);
        acessos = null;

        /*
         * O relatrio de frequencia  dividido em uma srie de pginas.
         *
         * 1. Gera a pgina de entrada
         */
        geraRelatorioFrequenciaEntrada(ve, periodos, dataInicio, dataFim);

        /*
         * 2. Gera uma tabela de acessos para cada perodo.
         *
         * Geramos duas vezes estas tabelas: na primeira, agrupamos os
         * participantes do curso de acordo com os grupos da ferramenta Grupos;
         * na segunda, criamos falsos grupos com Alunos, Formadores, Convidados
         * e Visitantes.
         */
        final List grupos = Grupo.todosGrupos(curso);

        Thread t0 = new Thread(new Runnable() {
            public void run() {
                try {
                    geraRelatorioFrequenciaPeriodos(ve, periodos, dataInicio,
                            dataFim, grupos, true);
                } catch (ResourceNotFoundException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (ParseErrorException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        });

        /*
         * geraRelatorioFrequenciaPeriodos(ve, periodos, dataInicio, dataFim,
         * grupos, true);
         */

        Grupo formadores = new Grupo();
        formadores.setNome("Formadores");
        Grupo alunos = new Grupo();
        alunos.setNome("Alunos");
        Grupo convidados = new Grupo();
        convidados.setNome("Convidados");
        Grupo visitantes = new Grupo();
        visitantes.setNome("Visitantes");
        Collection c = UsuarioFactory.getInstance(curso).todosUsuarios();
        List todosUsuarios = new ArrayList(c);
        Collections.sort(todosUsuarios, this.comparatorUsuario);

        for (Iterator itr = todosUsuarios.iterator(); itr.hasNext();) {
            Usuario usr = (Usuario) itr.next();
            if (usr.isAluno()) {
                alunos.addUsuario(usr);
            } else if (usr.isFormador()) {
                formadores.addUsuario(usr);
            } else if (usr.isConvidado()) {
                convidados.addUsuario(usr);
            } else if (usr.isVisitante()) {
                visitantes.addUsuario(usr);
            }
        }

        final List gruposFalsos = new ArrayList();
        gruposFalsos.add(alunos);
        gruposFalsos.add(formadores);
        gruposFalsos.add(convidados);
        gruposFalsos.add(visitantes);

        Thread t1 = new Thread(new Runnable() {
            public void run() {
                try {
                    geraRelatorioFrequenciaPeriodos(ve, periodos, dataInicio,
                            dataFim, gruposFalsos, false);
                } catch (ResourceNotFoundException e) {
                    // TODO Auto-generated catch block e.printStackTrace();
                } catch (ParseErrorException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (Exception e) { // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        });

        /*
         * geraRelatorioFrequenciaPeriodos(ve, periodos, dataInicio, dataFim,
         * gruposFalsos, false);
         */

        /*
         * 3. Gera uma pgina para cada dia do ms, com os acessos de cada
         * participante naquele dia
         */

        Thread t2 = new Thread(new Runnable() {
            public void run() {
                try {
                    geraRelatoriosDiarios(ve, dataInicio, dataFim);
                } catch (ResourceNotFoundException e) { // TODO Auto-generated
                    // catch block
                    e.printStackTrace();
                } catch (ParseErrorException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        });

        //geraRelatoriosDiarios(ve, dataInicio, dataFim);

        /*
         * 4. Gera uma pgina por perodo, por participante, com os acessos do
         * participante no perodo. Por uma questo de espao, vamos gerar para
         * somente 1 arquivo que lista todo os acessos do usurio, da entrada no
         * ambiente, passando por cada ferramenta que ele visitou
         */

        Thread t3 = new Thread(new Runnable() {
            public void run() {
                try {
                    geraRelatorioPeriodoParticipanteUsuarios(ve, periodos);
                } catch (ParseErrorException e) { // TODO Auto-generated catch
                    // block
                    e.printStackTrace();
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        });
        //geraRelatorioPeriodoParticipanteUsuarios(ve, periodos);

        t0.start();
        t1.start();
        t2.start();
        t3.start();

        t0.join();
        t1.join();
        t2.join();
        t3.join();

    }

    /**
     * @param ve
     * @param periodos
     * @param grupos
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     * @throws Exception
     * @throws ParseErrorException
     * @throws ParseErrorException
     */
    private void geraRelatorioPeriodoParticipanteUsuarios(VelocityEngine ve,
            Map periodos) throws ResourceNotFoundException,
            ParseErrorException, Exception {

        final Template template = ve
                .getTemplate("templates/acessos/relatorioPeriodoParticipante.vsl");
        /* VelocityContext context = super.createVelocityContext(); */
        DateTool dateTool = new DateTool();
        FerramentaTool ferramentaTool = new FerramentaTool();

        VelocityContext context = super.createVelocityContext();

        UsuarioFactory uf = UsuarioFactory.getInstance(curso);
        for (Iterator itr = uf.todosUsuarios().iterator(); itr.hasNext();) {
            Usuario usr = (Usuario) itr.next();

            for (Iterator i2 = periodos.keySet().iterator(); i2.hasNext();) {

                Date dataInicio = (Date) i2.next();
                Date dataFim = (Date) periodos.get(dataInicio);

                AcessosUsuario acessos = new AcessosUsuario(this.curso, usr,
                        dataInicio, dataFim);

                List acessosDia = acessos.getAcessos();
                if (!acessosDia.isEmpty()) {
                    context.put("acessosDia", acessosDia);
                    context.put("dataInicio", dataInicio);
                    context.put("dataFim", dataFim);
                    context.put("acessos", acessos);
                    context.put("usuarioAcessos", usr);
                    context.put("ferramentatool", ferramentaTool);

                    Writer writer = createWriter(this.dirExtracao
                            + "/relatorioPeriodoParticipante" + "_"
                            + dateTool.format("dd_MM_yyyy", dataInicio) + "_"
                            + dateTool.format("dd_MM_yyyy", dataFim) + "_usr_"
                            + usr.getId() + ".html");
                    mergeTemplate(template, context, writer);

                }
            }
        }
    }

    /**
     * @param dataFim
     * @param dataInicio
     * @param ve
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     *
     */
    private void geraRelatoriosDiarios(VelocityEngine ve, Date dataInicio,
            Date dataFim) throws ResourceNotFoundException,
            ParseErrorException, Exception {

        final Template template = ve
                .getTemplate("templates/acessos/relatorioDiario.vsl");
        final DateTool df = new DateTool();
        final FerramentaTool ferramentaTool = new FerramentaTool();

        VelocityContext context = super.createVelocityContext();
        Curso c = this.curso;
        Writer writer = null;
        String fileName = null;
        List dias = new Calendario(dataInicio, dataFim).toList();
        byte ferramenta[] = new FerramentaTool().geraIdFerramentas();

        for (int i = 0; i < ferramenta.length; i++) {
            final byte f = ferramenta[i];
            //final byte f = idFerramenta;
            for (Iterator itr = dias.iterator(); itr.hasNext();) {
                Date d = (Date) itr.next();

                context.put("data", d);

                AcessosFerramenta acessos = new AcessosFerramenta(c, f, d);
                List acessosDia = acessos.getAcessos(d);
                if (!acessosDia.isEmpty()) {
                    context.put("acessosDia", acessosDia);
                    context.put("acessos", acessos);
                    context.put("nomeFerramenta", ferramentaTool
                            .nomeFerramenta(f));

                    fileName = this.dirExtracao + "/relatorioDiario_"
                            + df.format("dd_MM_yyyy", d) + "_ferramenta_" + f
                            + ".html";
                    writer = createWriter(fileName);
                    mergeTemplate(template, context, writer);

                }

            }

        }
    }

    /**
     * Gera o relatrio de freqncia dos participantes do curso, dividido em
     * subperodos.
     *
     * @param VelocityEngine
     *            ve - utilizado para pegar templates.
     * @param Map
     *            periodos periodos - Os subperodos em que o curso est
     *            dividido.
     * @param Date
     *            dataInicio - Data de incio do curso
     * @param Date
     *            dataFim - Data do fim do curso
     * @param List
     *            grupos - List&lt;Grupo&gt;, pode conter os grupos do curso ou
     *            grupos artificiais, que contm Formadores, Alunos, Convidados
     *            e Visitantes
     * @param boolean
     *            gruposVerdadeiros - indica se os grupos na lista so grupos
     *            verdadeiros do curso ou se so artificiais.
     *
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     *
     * @see TransformAcessos.periodos
     *
     */
    private void geraRelatorioFrequenciaPeriodos(VelocityEngine ve,
            Map periodos, final Date dataInicio, final Date dataFim,
            List grupos, boolean gruposVerdadeiros)
            throws ResourceNotFoundException, ParseErrorException, Exception {

        VelocityContext context = super.createVelocityContext();

        final Locale locale = new Locale("pt");
        DateTool dateTool = new DateTool() {
            public Locale getLocale() {
                return locale;
            }
        };
        context.put("datetool", dateTool);

        final Template template = ve
                .getTemplate("templates/acessos/relatorioFrequenciaPeriodos.vsl");

        byte[] ferramentas = new FerramentaTool().geraIdFerramentas();

        context.put("calendartool", new CalendarTool());
        context.put("ferramentatool", new FerramentaTool());
        context.put("grupos", grupos);
        context.put("gruposVerdadeiros", new Boolean(gruposVerdadeiros));

        /*
         * periodo do inicio ao fim do curso
         */
        //Calendario calendario = new Calendario(dataInicio, dataFim);
        FerramentaTool ferramentaTool = new FerramentaTool();

        Boolean myFalse = new Boolean(false);

        Writer writer;

        for (int i = 0; i < ferramentas.length; i++) {
            byte idFerramenta = ferramentas[i];

            context.put("nomeFerramenta", ferramentaTool
                    .nomeFerramenta(idFerramenta));
            context.put("ferramenta", new Byte(idFerramenta));

            List l = new ArrayList(periodos.keySet());
            Date dataInicioAnterior = null, dataFimAnterior = null;

            for (int j = 0; j < l.size(); j++) {

                Date d1 = (Date) l.get(j);
                Date d2 = (Date) periodos.get(d1);
                Calendario cal = new Calendario(d1, d2);

                context.put("dataInicio", d1);
                context.put("dataFim", d2);
                context.put("calendario", cal.toList());
                context.put("meses", cal.meses());
                context.put("acessos", new AcessosFerramenta(this.curso,
                        idFerramenta, d1, d2));

                if (j + 1 < l.size()) {
                    Date dataInicioProximo = (Date) l.get(j + 1);
                    Date dataFimProximo = (Date) periodos
                            .get(dataInicioProximo);
                    context.put("dataInicioProx", dataInicioProximo);
                    context.put("dataFimProx", dataFimProximo);
                } else {
                    context.put("dataInicioProx", myFalse);
                    context.put("dataFimProx", myFalse);
                }

                context.put("dataInicioAnterior", dataInicioAnterior);
                context.put("dataFimAnterior", dataFimAnterior);

                final String f = nomeiaArquivoFrequencia(d1, d2,
                        gruposVerdadeiros, idFerramenta);

                writer = createWriter(f);
                mergeTemplate(template, context, writer);

                dataInicioAnterior = d1;
                dataFimAnterior = d2;
            }
        }

    }

    /**
     * @param dataInicio
     * @param dataFim
     * @param gruposVerdadeiros
     * @param dateTool
     * @param idFerramenta
     * @return
     */
    private String nomeiaArquivoFrequencia(Date dataInicio, Date dataFim,
            boolean gruposVerdadeiros, byte idFerramenta) {
        DateTool dateTool = new DateTool();
        StringBuffer sb = new StringBuffer(this.dirExtracao).append("/")
                .append("relatorioFrequenciaPeriodo").append("_").append(
                        dateTool.format("dd_MM_yyyy", dataInicio)).append("_")
                .append(dateTool.format("dd_MM_yyyy", dataFim)).append("_")
                .append(gruposVerdadeiros ? "grupos" : "semgrupos").append(
                        "_ferramenta_").append(idFerramenta).append(".html");
        return sb.toString();
    }

    /**
     * @param ve
     * @param periodos
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraRelatorioFrequenciaEntrada(VelocityEngine ve,
            Map periodos, Date dataInicio, Date dataFim)
            throws ResourceNotFoundException, ParseErrorException, Exception {
        Template template = ve
                .getTemplate("templates/acessos/relatorioFrequenciaEntrada.vsl");
        VelocityContext context = super.createVelocityContext();

        context.put("periodos", periodos);
        context.put("dataInicio", dataInicio);
        context.put("dataFim", dataFim);

        byte[] idFerramentas = new FerramentaTool().geraIdFerramentas();
        List ferramentas = new ArrayList(idFerramentas.length);
        for (int i = 0; i < idFerramentas.length; i++) {
            ferramentas.add(new Byte(idFerramentas[i]));
        }
        context.put("ferramentas", ferramentas);
        context.put("ferramentatool", new FerramentaTool());

        Writer writer = super.createWriter(this.dirExtracao
                + "/relatorioFrequencia.html");
        super.mergeTemplate(template, context, writer);
    }

    /**
     * Quebra o perodo entre duas datas em diversos perodos menores do tamanho
     * especificado. Estes perodos menores so retornados em um objeto Map
     * <Date,Date>, onde o Key e o Value so o incio e o fim de um sub-perodo.
     *
     * Quebra o perodo entre duas datas em diversos perodos menores do tamanho
     * especificado. Estes perodos menores so retornados em um objeto Map
     * <Date,Date>, onde o Key e o Value so o incio e o fim de um sub-perodo.
     * Todos os objetos Date no Key e no Value esto s 00:00 horas. Exemplo de
     * uso: <code>
     * Map m = divideSubPeriodos(d1, d2);
     * int i = 0;
     * for (Iterator itr = m.keySet().iterator(); itr.hasNext(); ) {
     *     Date dInicio = (Date) itr.next();
     *     Date dFim    = (Date) m.get(dInicio);
     *     System.out.println("Periodo " + i + ": de " + dInicio + " ate " + dFim);
     * }
     * </code>
     * O conjunto de chaves (keySet()) retornado est ordenado.
     *
     * @param dataInicio
     * @param dataFim
     * @return
     */
    private Map divideSubPeriodos(Date dataInicio, Date dataFim,
            int tamanhoSubPeriodo) {
        /*
         * Uso TreeMap ao invs de HashMap para que o conjunto de chaves esteja
         * ordenado.
         */
        Map periodos = new TreeMap();

        GregorianCalendar calendar = new GregorianCalendar();
        // Colocamos o fim do perodo no domingo.
        calendar.setTime(dataFim);
        while (calendar.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY) {
            calendar.add(Calendar.DAY_OF_WEEK, 1);
        }
        Date dFim = calendar.getTime();

        // Fazemos o primeiro perodo comear na segunda-feira
        calendar.setTime(dataInicio);
        while (calendar.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY) {
            calendar.add(Calendar.DAY_OF_WEEK, -1);
        }
        Date d1 = calendar.getTime();

        // Preenchemos o mapa com sub-perodos de duas semanas
        //DateFormat df = new SimpleDateFormat("dd-MM-yyyy");
        Date d2;
        do {
            calendar.add(Calendar.DAY_OF_MONTH, tamanhoSubPeriodo);
            d2 = calendar.getTime();
            periodos.put(d1, d2);

            calendar.add(Calendar.DAY_OF_MONTH, 1);
            d1 = calendar.getTime();
        } while (d1.compareTo(dFim) <= 0);
        return periodos;
    }

    /**
     * Gera a pgina de relatrio de acessos dos usurios, com data de ltimo
     * acesso e quantidade de acessos por usurio.
     *
     * @param context
     * @param template
     * @throws Exception
     * @throws MethodInvocationException
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraRelatorioAcessos(VelocityEngine ve, VelocityContext context)
            throws ResourceNotFoundException, ParseErrorException,
            MethodInvocationException, Exception {

        Template template = ve
                .getTemplate("templates/acessos/relatorioAcessos.vsl");

        context.put("mathtool", new MathTool());

        /*
         * Quantidade de acessos de cada usurio
         */
        context.put("acessos", new AcessosCurso(curso));

        /*
         * Lista dos usurios do curso, ordenada alfabeticamente
         */
        UsuarioFactory uf = UsuarioFactory.getInstance(this.curso);
        List listaUsuarios = new ArrayList(uf.todosUsuarios());

        Collections.sort(listaUsuarios, this.comparatorUsuario);
        context.put("listaUsuarios", listaUsuarios);

        Writer writer = super.createWriter(this.dirExtracao
                + "/relatorioAcessos.html");
        super.mergeTemplate(template, context, writer);
    }

    /**
     * Gera a pgina de entrada da ferramenta Acessos.
     *
     * @param context
     * @param template
     * @throws Exception
     * @throws MethodInvocationException
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraEntrada(VelocityEngine ve, VelocityContext context)
            throws IOException, ResourceNotFoundException, ParseErrorException,
            MethodInvocationException, Exception {
        Template template = ve.getTemplate("templates/acessos/entrada.vsl");

        Writer writer = super.createWriter(this.dirExtracao + "/index.html");
        super.mergeTemplate(template, context, writer);
    }
}