/*
 * Created on Oct 7, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.atividades;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.curso.Ferramenta;
import br.unicamp.nied.teleduc.extrator.database.ConnectionFactory;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.ferramentas.EnderecoWeb;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;

/**
 * @author leonel
 *
 */
public class Atividade extends Ferramenta {
    private String titulo;
    private String texto;
    private char tipoCompartilhamento;
    private int posicao;
    private int idAvaliacao = -1;

    public int getAvaliacao() {
        return idAvaliacao;
    }

    public void setAvaliacao(int idAvaliacao) {
        this.idAvaliacao = idAvaliacao;
    }

    public char getTipoCompartilhamento() {
        return tipoCompartilhamento;
    }

    public void setTipoCompartilhamento(char tipoCompartilhamento) {
        this.tipoCompartilhamento = tipoCompartilhamento;
    }

    protected String nomeFerramenta() {
        return "atividades";
    }

    public Atividade() {
        super();
    }

    public String getTitulo() {
        return this.titulo;
    }

    public void setTitulo(String t) {
        this.titulo = t;
    }

    public String getTexto() {
        return this.texto;
    }

    public void setTexto(String t) {
        this.texto = t;
    }

    public int getPosicao() {
        return posicao;
    }

    public void setPosicao(int posicao) {
        this.posicao = posicao;
    }

    public static PastaAtividades findByUsuario(Curso curso, Usuario usr)
            throws ExtractionException {

        PastaAtividades raiz = null;
        Map pastas = new HashMap();
        Map itens = new HashMap();

        String sql = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try {
            ConnectionFactory fac = ConnectionFactory.getConnectionFactory();
            Connection conn = fac.getConnection(curso.getBaseCurso());

            // 1. buscar as pastas e organizar a hierarquia
            sql = geraSQLPastas();
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();
            while (rs.next()) {
                /*
                 * Ao buscar uma pasta, verificamos se a pasta pai dela ja foi
                 * buscado anteriormente. Se nao tiver sido, criamos um pai
                 * "estepe" para pendurar esta. O pai de verdade sera buscado
                 * mais tarde.
                 */
                int id = rs.getInt("cod_topico");
                String nome = rs.getString("topico");
                int posicao = rs.getInt("posicao_topico");

                Integer i = new Integer(id);
                PastaAtividades p = (PastaAtividades) pastas.get(i);
                if (p == null) {
                    p = new PastaAtividades();
                    p.setId(id);
                    pastas.put(i, p);
                }
                p.setNome(nome);
                p.setData(rs.getLong("data"));
                p.setPosicao(posicao);

                if (id == 1) {
                    raiz = p;
                }

                int idPai = rs.getInt("cod_topico_pai");
                Integer iPai = new Integer(idPai);
                PastaAtividades pai = (PastaAtividades) pastas.get(iPai);
                if (pai == null) {
                    pai = new PastaAtividades();
                    pai.setId(idPai);
                    pastas.put(iPai, pai);
                }
                pai.addSubPasta(p);
            }
            try {
                rs.close();
                stmt.close();
            } catch (SQLException sqlex) {
                throw new ExtractionException(
                        "Erro ao buscar pasta atividades", sqlex);
            }

            // 2. Buscar os itens e anexa-los as pastas
            sql = geraSQLItensUsuario(usr);
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();
            while (rs.next()) {
                Atividade at = new Atividade();

                int id = rs.getInt("cod_item");
                Integer i = new Integer(id);

                at.setId(id);
                at.setAvaliacao(-1);
                at.setData(rs.getLong("data"));
                at.setTitulo(rs.getString("titulo"));
                at.setTexto(rs.getString("texto"));
                at.setPosicao(rs.getInt("posicao_item"));
                at.setTipoCompartilhamento(rs
                        .getString("tipo_compartilhamento").charAt(0));
//				at.setArquivos(at.retornaArquivos(curso));
                itens.put(i, at);

                int t = rs.getInt("cod_topico");
                PastaAtividades pai = (PastaAtividades) pastas.get(new Integer(
                        t));
                pai.addItem(at);
            }
            try {
                rs.close();
                stmt.close();
            } catch (SQLException sqlex) {
                throw new ExtractionException(
                        "Erro ao buscar itens de atividades", sqlex);
            }

            sql = geraSQLAvaliacao(usr);
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();
            while (rs.next()) {
                Atividade atividade = (Atividade) itens.get(new Integer(rs
                        .getInt("cod_item")));
                if (atividade != null) {
                    //Avaliacao aval = new Avaliacao();
                    //aval.setId(rs.getInt("cod_avaliacao"));
                    /*aval.setData(rs.getLong("data"));
                    aval.setDataInicio(rs.getLong("data_inicio"));
                    aval.setDataTermino(rs.getLong("data_termino"));
                    //aval.setDataInicio_br(rs.getString("data_inicio_br"));
                    //aval.setDataInicio_en(rs.getString("data_inicio_en"));
                    //aval.setDataTermino_br(rs.getString("data_termino_br"));
                    //aval.setDataTermino_en(rs.getString("data_termino_en"));
                    aval.setCriterios(rs.getString("criterios"));
                    aval.setObjetivos(rs.getString("objetivos"));
                    aval.setTipo(rs.getString("tipo").charAt(0));
                    aval.setValor(rs.getFloat("valor"));*/
                    atividade.setAvaliacao(rs.getInt("cod_avaliacao"));
                }
            }

            // 3. Buscar os enderecos web e anexa-los aos itens
            sql = geraSQLEnderecos(usr);
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();
            while (rs.next()) {
                EnderecoWeb end = new EnderecoWeb();
                end.setTitulo(rs.getString("nome"));
                end.setUrl(rs.getString("endereco"));

                int id = rs.getInt("cod_item");
                Integer i = new Integer(id);
                Atividade at = (Atividade) itens.get(i);
                at.addEnderecoWeb(end);
            }
            try {
                rs.close();
                stmt.close();
            } catch (SQLException sqlex) {
                throw new ExtractionException(
                        "Erro ao buscar enderecos web de atividades", sqlex);
            }
        } catch (IOException ioex) {
            throw new ExtractionException("Erro ao extrair Atividade da base",
                    ioex);
        } catch (SQLException sqlex) {
            throw new ExtractionException("Erro ao extrair Atividade da base",
                    sqlex);
        } catch (DBException dbex) {
            throw new ExtractionException("Erro ao extrair Atividade da base",
                    dbex);
        }

        return raiz;
    }

    /**
     * @param usr
     * @return
     */
    private static String geraSQLItensUsuario(Usuario usr) {
        String sql = null;
        if (usr.isFormador()) {
            // cod_topico = 0 --> excluido, = 2 --> lixeira
            sql = "select tipo_compartilhamento, cod_item, cod_topico, titulo, texto, posicao_item, data"
                    + " "
                    + "from Atividade_itens"
                    + " "
                    + "where cod_topico not in (0, 2)";
        } else if (usr.isAluno()) {
            sql = "select tipo_compartilhamento,cod_item, data, cod_topico, titulo, texto, posicao_item"
                    + " "
                    + "from Atividade_itens"
                    + " "
                    + "where cod_topico not in (0, 2) and tipo_compartilhamento='T'";
        } else {
            assert false : "Inicialmente, deve extrair apenas para alunos e formadores";
        }
        return sql;
    }

    private static String geraSQLAvaliacao(Usuario usr) {
        String sql = null;
        if (usr.isFormador()) {
            sql = "select A.tipo, A.data, A.data_inicio, A.data_termino, "
                    + "FROM_UNIXTIME(A.data_inicio,'%d/%m/%Y') as data_inicio_br, "
                    + "FROM_UNIXTIME(A.data_termino,'%d/%m/%Y') as data_termino_br, "
                    + "FROM_UNIXTIME(data_inicio,'%m/%d/%Y') as data_inicio_en, "
                    + "FROM_UNIXTIME(data_termino,'%m/%d/%Y') as data_termino_en, "
                    + "A.inicio_edicao, A.criterios, A.objetivos, "
                    + "A.valor, A.cod_avaliacao, I.cod_item from Atividade_itens as I, "
                    + "Avaliacao as A where A.cod_atividade=I.cod_item "
                    + "and A.ferramenta='P' and A.status!='X' and "
                    + "I.tipo_compartilhamento='T' and I.cod_topico "
                    + "not in (0, 2)";
        } else {
            sql = "select A.inicio_edicao, A.criterios, A.objetivos, "
                    + "A.tipo, A.valor, A.data, A.data_inicio, A.data_termino, "
                    + "FROM_UNIXTIME(A.data_inicio,'%d/%m/%Y') as data_inicio_br, "
                    + "FROM_UNIXTIME(A.data_termino,'%d/%m/%Y') as data_termino_br, "
                    + "FROM_UNIXTIME(data_inicio,'%m/%d/%Y') as data_inicio_en, "
                    + "FROM_UNIXTIME(data_termino,'%m/%d/%Y') as data_termino_en, "
                    + "A.cod_avaliacao, I.cod_item from "
                    + "Atividade_itens as I, Avaliacao as A "
                    + "where A.cod_atividade=I.cod_item and "
                    + "A.ferramenta='P' and A.status!='X' and"
                    + " I.tipo_compartilhamento='T' and "
                    + "I.cod_topico not in (0, 2)";
        }
        return sql;
    }

    /**
     * @param usr
     * @return
     */
    private static String geraSQLEnderecos(Usuario usr) {
        String sql = null;
        if (usr.isFormador()) {
            sql = "select AI.cod_item as cod_item, AIE.nome as nome, AIE.endereco as endereco"
                    + " from Atividade_itens as AI inner join Atividade_itens_enderecos AIE"
                    + " on AI.cod_item=AIE.cod_item"
                    + " where AI.cod_topico not in (0, 2)";
        } else if (usr.isAluno()) {
            sql = "select AI.cod_item as cod_item, AIE.nome as nome, AIE.endereco as endereco"
                    + " from Atividade_itens as AI inner join Atividade_itens_enderecos AIE"
                    + " on AI.cod_item=AIE.cod_item"
                    + " where AI.cod_topico not in (0, 2) and AI.tipo_compartilhamento='T'";
        } else {
            assert false : "Inicialmente deve extrair apenas para formadores e alunos";
        }
        return sql;
    }

    /**
     * @return
     */
    private static String geraSQLPastas() {
        String sql = "select cod_topico, cod_topico_pai, topico, data, posicao_topico"
                + " " + "from Atividade_topicos";
        return sql;
    }

    public int hashCode() {
        return this.id;
    }

    public boolean equals(Object obj) {
        if (obj instanceof Atividade) {
            Atividade at = (Atividade) obj;
            return at.id == this.id;
        }
        return false;
    }

}