/*
 * Created on Nov 4, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.avaliacao;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.curso.Ferramenta;
import br.unicamp.nied.teleduc.extrator.database.ConnectionFactory;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;

/**
 * @author bruno
 *
 */
public class Avaliacao extends Ferramenta {
    //private int id;

    private int idAtividade;

    private char ferramenta;

    private Date dataInicio;

    private Date dataTermino;

    private String titulo;

    private char tipo;

    private float valor;

    private String criterios;

    private String objetivos;

    protected String nomeFerramenta() {
        return "avaliacao";
    }

    public String getTitulo() {
        return titulo;
    }

    public void setTitulo(String titulo) {
        this.titulo = titulo;
    }

    public String getCriterios() {
        return criterios;
    }

    public void setCriterios(String criterios) {
        this.criterios = criterios;
    }

    public char getFerramenta() {
        return ferramenta;
    }

    public void setFerramenta(char ferramenta) {
        this.ferramenta = ferramenta;
    }

    /*public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }*/

    public int getIdAtividade() {
        return idAtividade;
    }

    public void setIdAtividade(int idAtividade) {
        this.idAtividade = idAtividade;
    }

    public String getObjetivos() {
        return objetivos;
    }

    public void setObjetivos(String objetivos) {
        this.objetivos = objetivos;
    }

    public char getTipo() {
        return tipo;
    }

    public void setTipo(char tipo) {
        this.tipo = tipo;
    }

    public float getValor() {
        return valor;
    }

    public void setValor(float valor) {
        this.valor = valor;
    }

    /**
     * @return Returns the dataInicio.
     */
    public Date getDataInicio() {
        return this.dataInicio;
    }

    /**
     * @deprecated Use setDataInicio(long nSegundos)
     * @param dataInicio
     *            The dataInicio to set.
     */
    public void setDataInicio(Date dataInicio) {
        this.dataInicio = dataInicio;
    }

    /**
     * Configura a data em java.util.Date, a partir do nmero de <b>segundos
     * </b> do UnixTime Assim: <code>
     * ...
     * java.sql.ResultSet rs = stmt.executeQuery("select data from Tabela");
     * int segundos = rs.getLong("data");
     * item.setDate(segundos);
     * ...
     * </code>
     *
     * @param data
     *            Data em <b>segundos </b> do UnixTime
     */
    public void setDataInicio(long nSegundos) {
        this.dataInicio = new Date(nSegundos * 1000L);
    }

    /**
     * @return Returns the dataTermino.
     */
    public Date getDataTermino() {
        return this.dataTermino;
    }

    /**
     * @param dataTermino
     *            The dataTermino to set.
     */
    public void setDataTermino(Date dataTermino) {
        this.dataTermino = dataTermino;
    }

    /**
     * Configura a data em java.util.Date, a partir do nmero de <b>segundos
     * </b> do UnixTime Assim: <code>
     * ...
     * java.sql.ResultSet rs = stmt.executeQuery("select data from Tabela");
     * int segundos = rs.getLong("data");
     * item.setDate(segundos);
     * ...
     * </code>
     *
     * @param data
     *            Data em <b>segundos </b> do UnixTime
     */
    public void setDataTermino(long nSegundos) {
        this.dataTermino = new Date(nSegundos * 1000L);
    }

    public boolean isExercicioIndividual() {
        return ((this.ferramenta == 'E') && (this.tipo == 'I'));
    }

    public boolean isExercicioGrupo() {
        return ((this.ferramenta == 'E') && (this.tipo == 'G'));
    }

    public boolean isPortfolioIndividual() {
        return ((this.ferramenta == 'P') && (this.tipo == 'I'));
    }

    public boolean isPortfolioGrupo() {
        return ((this.ferramenta == 'P') && (this.tipo == 'G'));
    }

    public boolean isForum() {
        return (this.ferramenta == 'F');
    }

    public boolean isBatepapo() {
        return (this.ferramenta == 'B');
    }

    /**
     * @deprecated
     * @see is( <i>NomeFerramenta </i>)Individual <br>
     *
     * <b>Descrio: </b> <br>
     * Indica se esta  uma avaliao de atividade individual
     * @return BOOLEAN
     */

    public boolean isIndividual() {
        return this.tipo == 'I';
    }

    /**
     * @deprecated Indica se esta  uma avaliao de atividade em grupo
     * @see is( <i>Ferramenta </i>)Individual <br>
     * @return
     */
    public boolean isGrupal() {
        return this.tipo == 'G';
    }

    public Avaliacao() {
        super();
    }

    public static List retornaAvaliacoes(Curso curso, Usuario usuario)
            throws ExtractionException {
        String sqlAvaliacoes = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        Map mapAvaliacoes = new HashMap();
        List avaliacoes = new LinkedList();

        try {
            ConnectionFactory fac = ConnectionFactory.getConnectionFactory();
            Connection conn = fac.getConnection(curso.getBaseCurso());

            //gera a lista inicial de avaliacoes
            geraListaAvaliacoes(usuario, mapAvaliacoes, avaliacoes, conn);

            // Preenchimento dos titulos das avaliacoes
            sqlAvaliacoes = sqlTituloAtividade(usuario);
            stmt = conn.prepareStatement(sqlAvaliacoes);
            rs = stmt.executeQuery();
            while (rs.next()) {
                int id = rs.getInt("cod_avaliacao");
                Avaliacao aval = (Avaliacao) mapAvaliacoes.get(new Integer(id));

                assert aval != null : "o map NAO pode retornar uma aval NULL";

                String titulo = rs.getString("titulo");
                if((titulo.equals("")) || (titulo == null))
                    aval.setTitulo("N&atilde;o Dispon&iacute;vel");
                else
                    aval.setTitulo(titulo);
            }
            sqlAvaliacoes = sqlTituloBatepapo();
            stmt = conn.prepareStatement(sqlAvaliacoes);
            rs = stmt.executeQuery();
            while (rs.next()) {
                int id = rs.getInt("cod_avaliacao");
                Avaliacao aval = (Avaliacao) mapAvaliacoes.get(new Integer(id));

                assert aval != null : "o map NAO pode retornar uma aval NULL";

                String assunto = rs.getString("assunto");
                if((assunto.equals("")) || (assunto == null))
                    aval.setTitulo("N&atilde;o Dispon&iacute;vel");
                else
                    aval.setTitulo(assunto);
            }

            sqlAvaliacoes = sqlTituloExercicio();
            stmt = conn.prepareStatement(sqlAvaliacoes);

            rs = stmt.executeQuery();
            while (rs.next()) {
                int id = rs.getInt("cod_avaliacao");
                Avaliacao aval = (Avaliacao) mapAvaliacoes.get(new Integer(id));

                assert aval != null : "o map NAO pode retornar uma aval NULL";

                String titulo = rs.getString("titulo");
                if((titulo.equals("")) || (titulo == null))
                    aval.setTitulo("N&atilde;o Dispon&iacute;vel");
                else
                    aval.setTitulo(titulo);
            }

            sqlAvaliacoes = sqlTituloForum();
            stmt = conn.prepareStatement(sqlAvaliacoes);
            rs = stmt.executeQuery();
            while (rs.next()) {
                int id = rs.getInt("cod_avaliacao");
                Avaliacao aval = (Avaliacao) mapAvaliacoes.get(new Integer(id));

                assert aval != null : "o map NAO pode retornar uma aval NULL";

                String nome = rs.getString("nome");
                if((nome.equals("")) || (nome == null))
                    aval.setTitulo("N&atilde;o Dispon&iacute;vel");
                else
                    aval.setTitulo(nome);
            }

        } catch (DBException dbex) {
            throw new ExtractionException("Erro ao extrair Avaliacao", dbex);
        } catch (IOException ioex) {
            throw new ExtractionException("Erro ao extrair Avaliacao da base",
                    ioex);
        } catch (SQLException sqlex) {
            throw new ExtractionException("Erro ao extrair Avaliacao da base",
                    sqlex);
        }
        return avaliacoes;
    }

    /**
     * Gera diferentes listas de avaliacoes de acordo com o usuario passado
     *
     * @param usuario
     * @param mapAval
     * @param listAval
     * @param conn
     * @throws SQLException
     */
    public static void geraListaAvaliacoes(Usuario usuario, Map mapAval,
            List listAval, Connection conn) throws SQLException {

        String sql = "";
        PreparedStatement stmt;
        ResultSet rs;

        if (usuario.isFormador()) {
            sql = "select cod_avaliacao, cod_atividade, cod_usuario, "
                    + "ferramenta, tipo, valor, status, "
                    + "data, data_inicio, data_termino, "
                    + "criterios, objetivos from Avaliacao where status='F' "
                    + "ORDER BY data_inicio";
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();

            populaAval(mapAval, listAval, rs);

        } else { // nao eh formador
            sql = "select A.cod_avaliacao, A.cod_atividade, A.cod_usuario, "
                    + "A.ferramenta, A.tipo, A.valor, A.status, "
                    + "A.data, A.data_inicio, A.data_termino, "
                    + "A.criterios, A.objetivos from Avaliacao as A, "
                    + "Atividade_itens as At where A.status='F' "
                    + "and A.ferramenta='P' and A.cod_atividade = At.cod_item "
                    + "and At.tipo_compartilhamento='T' ORDER BY data_inicio";
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();

            populaAval(mapAval, listAval, rs);

            sql = "select cod_avaliacao, cod_atividade, cod_usuario, "
                    + "ferramenta, tipo, valor, status, "
                    + "data, data_inicio, data_termino, "
                    + "criterios, objetivos from Avaliacao where status='F' "
                    + "and ferramenta!='P' ORDER BY data_inicio";
            stmt = conn.prepareStatement(sql);
            rs = stmt.executeQuery();

            populaAval(mapAval, listAval, rs);

            Comparator c1 = new Comparator(){
                public int compare(Object obj1, Object obj2) {
                    Avaliacao aval1 = (Avaliacao) obj1;
                    Avaliacao aval2 = (Avaliacao) obj2;
                    return aval1.getDataInicio().compareTo(aval2.getDataInicio());
                }
            };

            Collections.sort(listAval, c1);
        }
    }

    public static void populaAval(Map mapAval, List listAval, ResultSet rs)
            throws SQLException {
        while (rs.next()) {
            Avaliacao aval = new Avaliacao();
            if(rs.getString("criterios") == null)
                aval.setCriterios("N&atilde;o Dispon&iacute;vel");
            else
                aval.setCriterios(rs.getString("criterios"));

            if(rs.getString("objetivos") == null)
                aval.setObjetivos("N&atilde;o Dispon&iacute;vel");
            else
                aval.setObjetivos(rs.getString("objetivos"));

            aval.setData(rs.getLong("data"));
            aval.setDataInicio(rs.getLong("data_inicio"));
            aval.setDataTermino(rs.getLong("data_termino"));
            char ferramenta = rs.getString("ferramenta").charAt(0);
            aval.setFerramenta(ferramenta);
            if ((ferramenta == 'P') || (ferramenta == 'E')) {
                aval.setTipo(rs.getString("tipo").charAt(0));
            }
            aval.setId(rs.getInt("cod_avaliacao"));
            aval.setIdAtividade(rs.getInt("cod_atividade"));
            aval.setValor(rs.getFloat("valor"));

            mapAval.put(new Integer(rs.getInt("cod_avaliacao")), aval);
            listAval.add(aval);
        }
    }

    public static String sqlTituloForum() {
        String sql = "select F.nome,A.cod_avaliacao from Forum as F,"
                + "Avaliacao as A where A.ferramenta='F' and A.status='F' "
                + "and A.cod_atividade=F.cod_forum";
        return sql;
    }

    public static String sqlTituloAtividade(Usuario usuario) {
        String sql = "";

        if (usuario.isFormador()) {
            sql = "select A.cod_avaliacao, At.titulo from Atividade_itens"
                    + " as At, Avaliacao as A where At.cod_topico!=2 and "
                    + "At.cod_item=A.cod_atividade and A.status='F' and A.ferramenta='P' and "
                    + "At.status='L'";
        } else {
            sql = "select A.cod_avaliacao, At.titulo from Atividade_itens"
                    + " as At, Avaliacao as A where At.cod_topico!=2 and "
                    + "At.cod_item=A.cod_atividade and A.status='F' and A.ferramenta='P' and "
                    + "At.status='L' and At.tipo_compartilhamento='T'";
        }
        return sql;
    }

    public static String sqlTituloBatepapo() {
        String sql = "select A.cod_avaliacao,BA.assunto from "
                + "Batepapo_assuntos as BA, Avaliacao as A where "
                + "A.cod_atividade=BA.cod_assunto and A.status='F' and "
                + "A.ferramenta='B'";
        return sql;
    }

    public static String sqlTituloExercicio() {
        String sql = "select A.cod_avaliacao,E.titulo from Exercicios_modelo as E, "
                + "Avaliacao as A where A.cod_atividade=E.cod_modelo and "
                + "A.status='F' and A.ferramenta='E'";
        return sql;
    }
}