/*
 * Created on Oct 20, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.diario;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.database.ConnectionFactory;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;

/**
 * Representa o dirio de bordo de um usurio.
 *
 * @author leonel
 *
 */
public class Diario {
    private int idUsuario;
    private List itens;

    /**
     *
     */
    public Diario() {
        super();
        this.itens = new ArrayList();
        // TODO Auto-generated constructor stub
    }

    /**
     * @return Returns the idUsuario.
     */
    public int getId() {
        return idUsuario;
    }
    /**
     * @param id The id to set.
     */
    public void setId(int id) {
        this.idUsuario = id;
    }
    /**
     * @return Returns the itens.
     */
    public List getItens() {
        return itens;
    }
    /**
     * @param itens The itens to set.
     */
    public void setItens(List itens) {
        this.itens = itens;
    }
    /**
     *
     * @author leonel
     * @param item Item de diario a adicionar
     */
    public void addItem(ItemDiario item) {
        this.itens.add(item);
    }

    /**
     * Retorna List com todos os diarios do curso aos quais um usuario tem acesso
     *
     * @param usuario usuario que esta acessando
     * @return Os diarios aos quais este usuario tem acesso
     * @throws ExtractionException
     */
    public static List findByUsuario(Curso curso, Usuario usuario)
        throws ExtractionException {
        DiarioFactory diarioFactory = new DiarioFactory();
        Map mapItens = new HashMap();

        // Recursos para sql
        String sql = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try {
            ConnectionFactory f = ConnectionFactory.getConnectionFactory();
            Connection conn = f.getConnection(curso.getBaseCurso());

            sql  = geraSQLItensDiario(usuario);
            stmt = conn.prepareStatement(sql);
            rs   = stmt.executeQuery();

            /*
             * Primeiro, pegue todos os itens.
             * Os dirios que contm os itens sero criados naturalmente  medida que os itens so processados.
             * Para isso, usamos uma classe interna DiarioFactory
             */
            /*
             * Verificamos se o usurio para o qual estamos extraindo tem itens em seu dirio
             * Se no tiver, criamos um dirio artificial para ele, sem itens;
             * Esta chamada cria automaticamente um dirio para o usurio, ver DiarioFactory
             */
            diarioFactory.getDiario(usuario.getId());
            while (rs.next()) {
                int idItem    = rs.getInt("cod_item");
                // O diario eh atrelado a um usuario. Vc identifica um diario pelo usuario dono dele.
                int idDiario  = rs.getInt("cod_usuario");
                String titulo = rs.getString("titulo");
                String texto  = rs.getString("texto");
                char compartilhamento = rs.getString("tipo_compartilhamento").charAt(0);

                ItemDiario item = new ItemDiario();
                item.setId(idItem);
                item.setTitulo(titulo);
                item.setAnotacao(texto);
                item.setData(new Date(1000L * rs.getLong("data")));
                item.setCompartilhamento(compartilhamento);
                mapItens.put(new Integer(idItem), item);

                Diario diario = diarioFactory.getDiario(idDiario);
                diario.addItem(item);
            }
            rs.close();
            stmt.close();

            /*
             * Agora atrelamos os comentarios aos itens.
             */
            sql  = geraSQLComentarios(usuario);
            stmt = conn.prepareStatement(sql);
            rs   = stmt.executeQuery();
            while (rs.next()) {
                Integer idItem   = new Integer(rs.getInt("cod_item"));
                int idComentario = rs.getInt("cod_comentario");
                int idAutor      = rs.getInt("cod_comentarist");
                Date dataComent  = new Date(rs.getLong("data") * 1000L);
                String texto     = rs.getString("comentario");

                ItemDiario item  = (ItemDiario) mapItens.get(idItem);
                assert item != null:
                    "Comentario " + idComentario + " sem item de diario associado";

                Comentario c = new Comentario();
                c.setId(idComentario);
                c.setIdAutor(idAutor);
                c.setData(dataComent);
                c.setTexto(texto);

                item.addComentario(c);
            }
        } catch (DBException dbex) {
            dbex.printStackTrace();
            throw new ExtractionException("Erro ao extrair Diarios", dbex);
        } catch (SQLException sqlex) {
            sqlex.printStackTrace();
            throw new ExtractionException("Erro ao extrair Diarios", sqlex);
        } catch (IOException ioex) {
            ioex.printStackTrace();
            throw new ExtractionException("Erro ao extrair Diarios", ioex);
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException sqlex) { /* ignore */ }
            }

            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException sqlex) { /* ignore */ }
            }
        }

        /*
         * Retornamos os diarios
         */

        List ret = diarioFactory.getAllDiarios();
        return ret;
    }

    /**
     * @param usuario
     * @return
     */
    private static String geraSQLComentarios(Usuario usuario) {
        String sql = null;

        if (usuario.isFormador()) {
            sql =
                "select DC.cod_item, DC.cod_comentario, DC.cod_comentarist, DC.comentario, DC.data " +
                  "from Diario_itens as DI inner join Diario_comentarios as DC " +
                    "on DI.cod_item=DC.cod_item " +
                 "where DC.status='A' " +
                   "and ((DI.cod_usuario=" + usuario.getId() + " and DI.status='A') " +
                    "or DI.tipo_compartilhamento in ('T', 'F'))";
        } else if (usuario.isAluno()) {
            sql =
                "select DC.cod_item, DC.cod_comentario, DC.cod_comentarist, DC.comentario, DC.data " +
                  "from Diario_itens as DI inner join Diario_comentarios as DC " +
                    "on DI.cod_item=DC.cod_item " +
                 "where DC.status='A' " +
                   "and ((DI.cod_usuario=" + usuario.getId() + " and DI.status='A') " +
                    "or DI.tipo_compartilhamento='T')";
        } else {
            assert false : "Extracao inicialmente programada somente para alunos e formadores";
        }
        return sql;
    }

    /**
     * @param usuario
     * @return
     */
    private static String geraSQLItensDiario(Usuario usuario) {
        String sql;
        if (usuario.isFormador()) {
            sql =
                "select cod_item, cod_usuario, titulo, texto, data, tipo_compartilhamento " +
                  "from Diario_itens " +
                 "where (cod_usuario=" + usuario.getId() + " and status='A') " +
                    "or tipo_compartilhamento in ('T', 'F')";
        } else {
            sql =
                "select cod_item, cod_usuario, titulo, texto, data, tipo_compartilhamento " +
                  "from Diario_itens " +
                 "where (cod_usuario=" + usuario.getId() + " and status='A') " +
                    "or tipo_compartilhamento='T'";
        }
        return sql;
    }

    /**
     *
     * @author leonel
     *
     */
    private static class DiarioFactory {
        private Map mapDiarios;
        private DiarioFactory() {
            mapDiarios = new HashMap();
        }

        private Diario getDiario(int idDiario) {
            Integer i = new Integer(idDiario);
            Diario d = (Diario) this.mapDiarios.get(i);
            if (d == null) {
                d = new Diario();
                d.setId(idDiario);
                this.mapDiarios.put(i, d);
            }
            return d;
        }

        /**
         *
         * @return
         */
        private List getAllDiarios() {
            /*
             * Precisamos retornar um list. Construimos um ArrayList sobre a Collection retornada pelo Map.
             */
            return new ArrayList(this.mapDiarios.values());
        }
    }

    /**
     *
     */
    public int hashCode() {
        return this.idUsuario;
    }
}
