    /*
 * Created on Jan 10, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.grupos;

import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import br.unicamp.nied.teleduc.extrator.config.Config;
import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;
import br.unicamp.nied.teleduc.transformador.Transform;

/**
 * @author andrey
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class TransformGrupos extends Transform {
    private Collection grupos;

    private Comparator compUsuario = new Comparator() {
        public int compare(Object obj1, Object obj2) {
            Usuario u1 = (Usuario) obj1;
            Usuario u2 = (Usuario) obj2;

            return u1.getNome().compareToIgnoreCase(u2.getNome());
        }
    };

    public TransformGrupos(Curso curso, Usuario usuario) throws IOException {
        super(curso, usuario, "grupos");
        File diretorioExtracao = new File(dirExtracao);
        if(!diretorioExtracao.exists()) {
            diretorioExtracao.mkdirs();
        }
    }

    public TransformGrupos(Curso curso, Usuario usuario, Collection grupos) throws IOException {
        super(curso, usuario, "grupos");
        File diretorioExtracao = new File(dirExtracao);
        this.grupos = grupos;
        if(!diretorioExtracao.exists()) {
            diretorioExtracao.mkdirs();
        }
    }

    /**
     * Funo inicial que chama ao outras para gerar os arquivos HTML referentes  ferramenta grupos
     */
    public void geraArquivosHTML() throws IOException, Exception {
        VelocityEngine ve = new VelocityEngine();
        ve.init(Config.getConfig().getProperties());

        /* Gera a lista de todos os grupos pertencentes a um determinado curso */
        geraListaGruposFormados(ve);

        /* Gera a lista dos componentes de um determinado grupo */
        geraListaComponentesGrupo(ve);

        /* Gera a lista de todos os componentes de todos os grupos */
        geraListaTodosComponentesGrupos(ve);

        /* Gera a lista dos grupos dos usuarios */
        geraListaGruposDosComponentes(ve);
    }

    /**
     * Gera a lista de todos os grupos pertencentes a um determinado curso
     *
     * @param ve
     * @throws IOException
     * @throws ExtractionException
     * @throws Exception
     */
    private void geraListaGruposFormados(VelocityEngine ve) throws IOException, ExtractionException, Exception {
        VelocityContext context = createVelocityContext();
        Writer writer = createWriter(this.dirExtracao + "/index.html");

        context.put("grupos", grupos);

        try {
            Template t = ve.getTemplate("templates/grupos/listaGruposFormados.vsl");
            t.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) { /* ignore */ }
        }
    }

    /**
     * Gera a lista dos componentes de um determinado grupo
     *
     * @param ve
     * @throws IOException
     * @throws ExtractionException
     * @throws Exception
     */
    private void geraListaComponentesGrupo(VelocityEngine ve) throws IOException, ExtractionException, Exception {
        VelocityContext context = createVelocityContext();

        for (Iterator itr = grupos.iterator(); itr.hasNext();) {
            Grupo grupo = (Grupo) itr.next();
            List usuarios = grupo.getUsuarios();

            Writer writer = createWriter(this.dirExtracao + "/listaComponentesGrupo_" + grupo.getId() + ".html");

            context.put("usuarios", usuarios);
            context.put("grupo", grupo);

            try {
                Template t = ve.getTemplate("templates/grupos/listaComponentesGrupo.vsl");
                t.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) { /* ignore */ }
            }
        }
    }

    /**
     * Gera a lista de todos os componentes de todos os grupos
     *
     * @param ve
     * @throws ExtractionException
     * @throws IOException
     * @throws Exception
     */
    private void geraListaTodosComponentesGrupos(VelocityEngine ve) throws ExtractionException, IOException, Exception {
        VelocityContext context = createVelocityContext();
        Writer writer = createWriter(this.dirExtracao + "/listaTodosComponentesGrupos.html");

        context.put("grupos", grupos);

        try {
            Template t = ve.getTemplate("templates/grupos/listaTodosComponentesGrupos.vsl");
            t.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) { /* ignore */ }
        }
    }

    /**
     * Gera a lista dos grupos dos usuarios
     *
     * @param ve
     * @throws IOException
     * @throws ExtractionException
     * @throws Exception
     */
    private void geraListaGruposDosComponentes(VelocityEngine ve) throws IOException, ExtractionException, Exception {
        VelocityContext context = createVelocityContext();
        Writer writer = createWriter(this.dirExtracao + "/listaGruposDosComponentes.html");

        Collection usuarios = UsuarioFactory.getInstance(curso).todosUsuarios();
        List usuariosAlunos = new ArrayList();
        List usuariosFormadores = new ArrayList();
        List usuariosOrdenados = new ArrayList();

        /* Ordenamos a lista de usuarios em ordem alfabtica, primeiro em relao ao tipo (aluno ou formador),
         * depois ao nome do usurio
         */
        for (Iterator itr = usuarios.iterator(); itr.hasNext();) {
            Usuario usr = (Usuario) itr.next();
            if (usr.isAluno()) {
                usuariosAlunos.add(usr);
            } else {
                usuariosFormadores.add(usr);
            }
        }

        Collections.sort(usuariosAlunos, compUsuario);
        Collections.sort(usuariosFormadores, compUsuario);

        usuariosOrdenados.addAll(usuariosAlunos);
        usuariosOrdenados.addAll(usuariosFormadores);

        context.put("usuarios", usuariosOrdenados);
        context.put("grupos", grupos);

        try {
            Template t = ve.getTemplate("templates/grupos/listaGruposDosComponentes.vsl");
            t.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) { /* ignore */ }
        }
    }
}
