/*
 * Created on Dec 16, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.leituras;

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.curso.Pasta;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.transformador.Transform;

/**
 * @author pablo
 *
 */
public class TransformLeituras extends Transform {
    private Pasta raizLeituras;

    /**
     * @param curso
     * @param usuario
     * @param nomeFerramenta
     * @throws IOException
     * @throws ExtractionException
     */
    public TransformLeituras(Curso curso, Usuario usuario) throws IOException,
            ExtractionException {
        this(curso, usuario, Leitura.retornaEstrutura(curso, usuario));
    }

    public TransformLeituras(Curso curso, Usuario usuario, Pasta raizLeituras)
            throws IOException {
        super(curso, usuario, "leituras");
        this.raizLeituras = raizLeituras;
    }

    public void geraArquivosHTML() throws IOException,
            ResourceNotFoundException, ParseErrorException, Exception {
        VelocityEngine ve = super.createVelocityEngine();
        VelocityContext context = super.createVelocityContext();

        /*
         * Gerar arquivos HTML para pastas de leituras
         */
        Template t = ve.getTemplate("templates/leituras/pastaLeituras.vsl");
        geraHTMLPastas(context, t, this.raizLeituras);

        /*
         * Gerar arquivos HTML para itens de leituras
         */
        t = ve.getTemplate("templates/leituras/itemLeituras.vsl");
        geraHTMLItens(context, t, this.raizLeituras);
    }

    private void geraHTMLPastas(VelocityContext context, Template template,
            Pasta raizLeituras) throws IOException, ResourceNotFoundException,
            ParseErrorException, MethodInvocationException, Exception {
        context.put("raiz", raizLeituras);
        geraHTMLPastasAux(context, template, raizLeituras, raizLeituras);
    }

    private void geraHTMLPastasAux(VelocityContext context, Template template,
            Pasta raizLeituras, Pasta pasta) throws IOException,
            ResourceNotFoundException, ParseErrorException,
            MethodInvocationException, Exception {
        /*
         * Gerar HTML para a pasta
         */
        Writer writer = null;
        if (pasta == raizLeituras) {
            writer = createWriter(this.dirExtracao + "/index.html");
        } else {
            writer = createWriter(this.dirExtracao + "/pastaLeituras_"
                    + pasta.getId() + ".html");
        }
        context.put("pasta", pasta);
        context.put("caminho", caminho(raizLeituras, pasta));
        try {
            template.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) {
                /* ignore */
            }
        }
        /*
         * Aplicar recursivamente para cada subpasta
         */
        for (Iterator itr = pasta.getSubPastas().iterator(); itr.hasNext();) {
            Pasta p = (Pasta) itr.next();
            geraHTMLPastasAux(context, template, raizLeituras, p);
        }
    }

    /**
     * @param raiz
     * @param pasta
     * @return
     */
    private List caminho(Pasta raiz, Pasta pasta) {
        LinkedList caminho = new LinkedList();
        caminhoAux(raiz, pasta, caminho);
        return caminho;
    }

    private boolean caminhoAux(Pasta p1, Pasta p2, LinkedList l) {
        if (p1 == p2) {
            l.addFirst(p2);
            return true;
        } else {
            for (Iterator itr = p1.getSubPastas().iterator(); itr.hasNext();) {
                Pasta p = (Pasta) itr.next();
                if (caminhoAux(p, p2, l)) {
                    l.addFirst(p1);
                    return true;
                }
            }
            return false;
        }
    }

    /**
     * @param context
     * @param t
     * @param pasta
     * @throws Exception
     * @throws IOException
     * @throws MethodInvocationException
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraHTMLItens(VelocityContext context, Template template,
            Pasta raiz) throws ResourceNotFoundException, ParseErrorException,
            MethodInvocationException, IOException, Exception {
        context.put("raiz", raiz);
        geraHTMLItensAux(context, template, raiz, raiz);
    }

    /**
     * @param context
     * @param template
     * @param raiz
     * @throws Exception
     * @throws IOException
     * @throws MethodInvocationException
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void geraHTMLItensAux(VelocityContext context, Template template,
            Pasta raiz, Pasta pasta) throws ResourceNotFoundException,
            ParseErrorException, MethodInvocationException, IOException,
            Exception {
        /*
         * Gerar arquivos HTML para os itens desta pasta
         */
        context.put("pasta", pasta);
        context.put("caminho", caminho(raiz, pasta));
        for (Iterator itr = pasta.getItens().iterator(); itr.hasNext();) {
            Leitura leitura = (Leitura) itr.next();
            geraItem(context, template, leitura);
        }

        /*
         * Gerar recursivamente arquivos HTML para os itens das subpastas
         */
        for (Iterator itr = pasta.getSubPastas().iterator(); itr.hasNext();) {
            Pasta p = (Pasta) itr.next();
            geraHTMLItensAux(context, template, raiz, p);
        }
    }

    /**
     * @param context
     * @param template
     * @param parada
     * @throws Exception
     * @throws MethodInvocationException
     * @throws ParseErrorException
     * @throws IOException
     * @throws ResourceNotFoundException
     */
    private void geraItem(VelocityContext context, Template template,
            Leitura leitura) throws IOException, ResourceNotFoundException,
            ParseErrorException, MethodInvocationException, Exception {
        context.put("leitura", leitura);
        Writer writer = createWriter(this.dirExtracao + "/itemLeituras_"
                + leitura.getId() + ".html");
        try {
            template.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) {
                /* ignore */
            }
        }
    }
}