/*
 * Created on Feb 1, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.perfil;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.curso.ExtratorFerramenta;
import br.unicamp.nied.teleduc.extrator.ferramentas.Anexo;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;

/**
 * @author leonel
 *
 */
public class ExtratorPerfil extends ExtratorFerramenta {
    /**
     * @param curso
     * @param usuario
     */
    public ExtratorPerfil(Curso curso, Usuario usuario) {
        super(curso, usuario);
    }

    public String nomeFerramenta() {
        return "perfil";
    }

    /**
     * Copia os arquivos da ferramenta Perfil. Todos os arquivos esto em um
     * mesmo diretrio, no h separao por itens como nas outras ferramentas.
     *
     */
    private Collection copiaArquivosPerfil() throws ExtractionException {
        try {
            String strDirOrigem = super.retornaCaminho();
            String strDirDestino = montaCaminhoDestino();
            File dirOrigem = new File(strDirOrigem);
            File dirDestino = new File(strDirDestino);
            return copiaDiretorio(dirOrigem, dirDestino);
        } catch (IOException ioex) {
            throw new ExtractionException("Erro ao copiar arquivos!", ioex);
        }
    }

    /**
     * @throws IOException
     *
     */
    public void extrair() throws ExtractionException {
        /*
         * Os arquivos da ferramenta Perfil so as fotografias anexadas pelos
         * usurios. Copiamos estes arquivos.
         */
        Collection fotos = this.copiaArquivosPerfil();
        Map hashFotos = new HashMap(fotos.size());
        for (Iterator itr = fotos.iterator(); itr.hasNext();) {
            Anexo foto = (Anexo) itr.next();
            Integer idUsr = reconheceId(foto.getName());
            hashFotos.put(idUsr, foto);
        }

        /*
         * Associamos a cada usurio o seu perfil.
         * Se o usurio no tiver perfil, colocamos um perfil vazio.
         */
        UsuarioFactory factory = UsuarioFactory.getInstance(curso);
        Map mapPerfis = Perfil.perfis(curso);
        for (Iterator itr = factory.todosUsuarios().iterator(); itr.hasNext();) {
            Usuario usr = (Usuario) itr.next();
            Integer id = new Integer(usr.getId());

            Perfil perfil = (Perfil) mapPerfis.get(id);
            if (perfil == null) {
                perfil = new Perfil();
                perfil.setId(id.intValue());
            }
            Anexo foto = (Anexo) hashFotos.get(id);
            usr.setPerfil(perfil);
            perfil.setFoto(foto);
        }

        /*
         * Geramos os arquivos HTML correspondentes s telas da ferramenta
         * Perfil
         */
        try {
            TransformPerfil trans = new TransformPerfil(this.curso,
                    this.usuario, factory);
            trans.geraArquivosHTML();
        } catch (IOException e) {
            e.printStackTrace();
            throw new ExtractionException(
                    "Erro ao gerar arquivos HTML da ferramenta perfil", e);
        } catch (Exception e) {
            e.printStackTrace();
            throw new ExtractionException(
                    "Erro ao gerar arquivos HTML da ferramenta perfil", e);
        }
    }

    /**
     * Recebe uma String com o nome da foto no formato "cod_usuario_XX.*", onde
     * XX  um inteiro representando o cdigo do usurio dono da foto, e retorna
     * o Integer correspondente.
     *
     * @param name
     * @return
     */
    private Integer reconheceId(String strFoto) {
        /*
         * Extraimos o inteiro da String utilizando uma expresso regular
         */
        String regex = "\\d+";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(strFoto);
        matcher.find();
        String strIdUsr = matcher.group();
        Integer idUsr = new Integer(strIdUsr);

        return idUsr;
    }
}