/*
 * Created on Dec 6, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.perfil;

import java.io.IOException;
import java.io.Writer;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;
import br.unicamp.nied.teleduc.transformador.Transform;

public class TransformPerfil extends Transform {
    private UsuarioFactory usuarioFactory;

    /**
     * @param curso
     * @param usuario
     * @param nomeFerramenta
     * @throws ExtractionException
     * @throws IOException
     * @throws DBException
     * @throws SQLException
     * @throws IOException
     */
    public TransformPerfil(Curso curso, Usuario usuario) throws IOException,
            ExtractionException {
        this(curso, usuario, UsuarioFactory.getInstance(curso));
    }

    public TransformPerfil(Curso curso, Usuario usuario, UsuarioFactory factory)
            throws IOException {
        super(curso, usuario, "perfil");
        this.usuarioFactory = factory;
    }

    public void geraArquivosHTML() throws ResourceNotFoundException,
            ParseErrorException, IOException, Exception {
        VelocityEngine ve = createVelocityEngine();
        VelocityContext context = createVelocityContext();

        Template template = ve.getTemplate("templates/perfil/listaPerfil.vsl");

        /*
         * Gerar HTML para lista de perfis de usurio
         */
        geraHTMLListaPerfil(context, template);

        /*
         * Gerar HTML para cada perfil individual de aluno
         */
        geraHTMLPerfisIndividuais(ve);
    }

    /**
     * @param ve
     * @throws ResourceNotFoundException
     * @throws ParseErrorException
     * @throws Exception
     * @throws ExtractionException
     * @throws IOException
     * @throws MethodInvocationException
     */
    private void geraHTMLPerfisIndividuais(VelocityEngine ve)
            throws ResourceNotFoundException, ParseErrorException, Exception,
            ExtractionException, IOException, MethodInvocationException {
        VelocityContext context = createVelocityContext();
        Template template = ve
                .getTemplate("templates/perfil/perfilUsuario.vsl");

        Writer writer = null;
        context.put("exibirCoordenador",new Boolean(curso.exibirCoordenador()));
        for (Iterator itr = this.usuarioFactory.todosUsuarios().iterator();
                itr.hasNext();) {

            Usuario usr = (Usuario) itr.next();
            context.put("usuarioPerfil", usr);
            try {
                writer = createWriter(this.dirExtracao + "/perfil_" + usr.getId()
                        + ".html");
                template.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) {
                    ioex.printStackTrace();
                    // ignore
                }
            }
        }
    }

    /**
     * @param context
     * @param template
     * @throws ExtractionException
     * @throws IOException
     * @throws ResourceNotFoundException
     * @throws ParseErrorException
     * @throws MethodInvocationException
     * @throws Exception
     */
    private void geraHTMLListaPerfil(VelocityContext context, Template template)
            throws ExtractionException, IOException, ResourceNotFoundException,
            ParseErrorException, MethodInvocationException, Exception {
        Collection todosUsuarios = this.usuarioFactory.todosUsuarios();
        Usuario coordenador = null;
        List alunos     = new ArrayList(todosUsuarios.size());
        List formadores = new ArrayList(todosUsuarios.size());
        List convidados = new ArrayList(todosUsuarios.size());
        List visitantes = new ArrayList(todosUsuarios.size());

        for (Iterator itr = todosUsuarios.iterator(); itr.hasNext();) {
            Usuario usr = (Usuario) itr.next();
            if (usr.isAluno()) {
                alunos.add(usr);
            } else if (usr.isFormador()) {
                formadores.add(usr);
                if (usr.isCoordenador()) {
                    coordenador = usr;
                }
            } else if (usr.isConvidado()) {
                convidados.add(usr);
            } else if (usr.isVisitante()) {
                visitantes.add(usr);
            } else {
                // ignore, ele no ser listado no perfil
            }
        }
        context.put("alunos", alunos);
        context.put("formadores", formadores);
        context.put("convidados", convidados);
        context.put("visitantes", visitantes);
        context.put("coordenador", coordenador);
        context.put("exibirCoordenador",new Boolean(curso.exibirCoordenador()));
        Writer writer = null;
        try {
            writer = createWriter(this.dirExtracao + "/index.html");
            template.merge(context, writer);
        } finally {
            if (writer != null) {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) {
                    ioex.printStackTrace();
                    // ignore
                }
            }
        }
    }
}