/*
 * Created on Jan 15, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.portfolio;

import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import br.unicamp.nied.teleduc.extrator.config.Config;
import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.ferramentas.grupos.Grupo;
import br.unicamp.nied.teleduc.extrator.ferramentas.grupos.GrupoFactory;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;
import br.unicamp.nied.teleduc.transformador.Transform;

/**
 * @author andrey
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class TransformPortfolio extends Transform {
    private List portfoliosIndividuais;
    private List portfoliosGrupo;

    private Comparator compData = new Comparator() {
        public int compare(Object obj1, Object obj2) {
            Comentario c1 = (Comentario) obj1;
            Comentario c2 = (Comentario) obj2;
            return -1 * c1.getData().compareTo(c2.getData());
        }
    };

    /**
     * @throws IOException
     * @throws ExtractionException
     */
    public TransformPortfolio(Curso curso, Usuario usuario) throws IOException, ExtractionException {
        super(curso, usuario, "portfolio");
        File diretorioExtracao = new File(dirExtracao);
        if(!diretorioExtracao.exists()) {
            diretorioExtracao.mkdirs();
        }
    }

    public TransformPortfolio(Curso curso, Usuario usuario, List portfoliosIndividuais, List portfoliosGrupo)
            throws IOException {
        super(curso, usuario, "portfolio");
        this.portfoliosIndividuais = portfoliosIndividuais;
        this.portfoliosGrupo = portfoliosGrupo;
    }

    public void geraArquivosHTML() throws Exception {
        VelocityEngine ve = new VelocityEngine();
        ve.init(Config.getConfig().getProperties());

        /* 1. Lista de portflios individuais */
        geraListaPortfoliosIndividuais(ve);

        /* 2. Lista de portflios de grupo */
        geraListaPortfoliosGrupo(ve);

        /* 3. Gerar exibio das pastas e itens de um portfolio */
        geraPortfolioIndividualouGrupo(ve);

        /* 4. Gera a pagina de componentes por grupo */
        geraComponentesGrupo(ve);
    }

    /**
     * Responsvel por gerar a lista de portflios de grupo
     *
     * @param ve
     * @throws ResourceNotFoundException
     * @throws ParseErrorException
     * @throws Exception
     */
    private void geraListaPortfoliosGrupo(VelocityEngine ve) throws ResourceNotFoundException, ParseErrorException, Exception {

        VelocityContext context = createVelocityContext();
        Writer writer = createWriter(this.dirExtracao + "/listaPortfoliosGrupo.html");

        context.put("tipo", "GRUPO");
        context.put("portfolios", portfoliosGrupo);
        context.put("todosGrupos", GrupoFactory.getInstance(this.curso));

        try {
            Template t = ve.getTemplate("templates/portfolio/listaPortfolios.vsl");
            t.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) { /* ignore */ }
        }
    }

    /**
     * Responsvel por gerar a lista de portflios individuais
     *
     * @param ve
     * @throws ResourceNotFoundException
     * @throws ParseErrorException
     * @throws Exception
     */
    private void geraListaPortfoliosIndividuais(VelocityEngine ve)
            throws ResourceNotFoundException, ParseErrorException, Exception {

        VelocityContext context = createVelocityContext();
        Writer writer = createWriter(this.dirExtracao + "/index.html");

        context.put("tipo", "INDIVIDUAL");
        context.put("portfolios", portfoliosIndividuais);
        context.put("todosUsuarios", UsuarioFactory.getInstance(curso));

        try {
            Template t = ve.getTemplate("templates/portfolio/listaPortfolios.vsl");
            t.merge(context, writer);
        } finally {
            try {
                writer.flush();
                writer.close();
            } catch (IOException ioex) { /* ignore */ }
        }
    }

    /**
     * Chama as funes para gerar o portflio Individual e de Grupo
     *
     * @param ve
     * @throws ExtractionException
     * @throws ParseErrorException
     * @throws Exception
     */
    private void geraPortfolioIndividualouGrupo(VelocityEngine ve)
        throws ExtractionException, ParseErrorException, Exception {

        Template template = ve.getTemplate("templates/portfolio/portfolio.vsl");

        /* Gera portfolio individual */
        geraPortfolio(ve, template, this.portfoliosIndividuais, "INDIVIDUAL");

        /* Gera portfolio de grupo */;
        geraPortfolio(ve, template, this.portfoliosGrupo, "GRUPO");

    }

    /**
     * Gera tanto o portfolio individual como de grupo de acordo com a string tipo
     *
     * @param ve
     * @param t
     * @param portfolios
     * @param tipo responsavel pela escolha do tipo de portfolio que sera gerado (INDIVIDUAL OU GRUPO)
     * @throws ExtractionException
     * @throws IOException
     * @throws Exception
     */
    private void geraPortfolio(VelocityEngine ve, Template t, List portfolios, String tipo)
        throws ExtractionException, IOException, Exception {

        VelocityContext context = createVelocityContext();
        Writer writer;

        for (Iterator itr = portfolios.iterator(); itr.hasNext();) {
            Portfolio portfolio = (Portfolio) itr.next();

            PastaPortfolio raiz = (PastaPortfolio) portfolio;
            List caminho = caminho(raiz, raiz);

            Usuario usuarioA = null;

            if (tipo.equals("INDIVIDUAL")) {
                writer = createWriter(this.dirExtracao + "/portfolioIndividual_"+ portfolio.getIdProprietario() +".html");
                usuarioA = UsuarioFactory.getInstance(curso).getUsuario(portfolio.getIdProprietario());
                context.put("usuarioAtual", usuario);
            } else { /* tipo == "GRUPO */
                writer = createWriter(this.dirExtracao + "/portfolioGrupo_"+ portfolio.getIdProprietario() +".html");
            }

            geraItensPortfolio(ve, portfolio.getItens(), usuarioA, raiz, raiz, tipo, portfolio);
            geraSubPastasPortfolio(ve, t, portfolio.getSubPastas(), usuarioA, raiz, tipo, portfolio);

            context.put("tipo", tipo);
            context.put("portfolio", portfolio);
            context.put("raiz", raiz);
            context.put("usuarioAtual", usuarioA);
            context.put("pasta", raiz);
            context.put("caminho", caminho);
            context.put("todosGrupos", GrupoFactory.getInstance(curso));
            context.put("todosUsuarios", UsuarioFactory.getInstance(curso));
            context.put("todasSubPastas", portfolio.getSubPastas());
            context.put("todosItens", portfolio.getItens());

            try {
                t.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) { /* ignore */ }
            }
        }
    }

    /**
     * Gera as subpastas de um portfolio
     *
     * @param ve
     * @param t
     * @param subPastas
     * @param usuario
     * @param raiz
     * @throws ExtractionException
     * @throws IOException
     * @throws Exception
     */
    private void geraSubPastasPortfolio(VelocityEngine ve, Template t, List subPastas, Usuario usuarioAtual, PastaPortfolio raiz, String tipo, Portfolio portfolio)
        throws ExtractionException, IOException, Exception {

        VelocityContext context = createVelocityContext();

        for (Iterator itr = subPastas.iterator(); itr.hasNext();) {
            PastaPortfolio pasta = (PastaPortfolio) itr.next();

            List subPastas2 = pasta.getSubPastas();
            List itens = pasta.getItens();

            List caminho = caminho(raiz, pasta);
            context.put("caminho", caminho);

            geraItensPortfolio(ve, itens, usuarioAtual, raiz, pasta, tipo, portfolio);
            geraSubPastasPortfolio(ve, t, subPastas2, usuarioAtual, raiz, tipo, portfolio);

            context.put("tipo", tipo);
            context.put("raiz", raiz);
            context.put("pasta", pasta);
            context.put("portfolio", portfolio);
            context.put("usuarioAtual", usuarioAtual);
            context.put("todosGrupos", GrupoFactory.getInstance(curso));
            context.put("todosUsuarios", UsuarioFactory.getInstance(curso));
            context.put("todasSubPastas", subPastas2);
            context.put("todosItens", itens);

            Writer writer = createWriter(this.dirExtracao + "/pastaPortfolio_"+ pasta.getId() +".html");

            try {
                t.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) { /* ignore */ }
            }
        }
    }

    /**
     * Gera os itens de todos os portfolios de um curso
     *
     * @param ve
     * @param itens
     * @param usuario
     * @param raiz
     * @throws ResourceNotFoundException
     * @throws ParseErrorException
     * @throws Exception
     */
    private void geraItensPortfolio(VelocityEngine ve, List itens, Usuario usuarioAtual, PastaPortfolio raiz, PastaPortfolio pasta, String tipo, Portfolio portfolio)
            throws ResourceNotFoundException, ParseErrorException, Exception {
        VelocityContext context = createVelocityContext();
        Template t = ve.getTemplate("templates/portfolio/itemPortfolio.vsl");

        for (Iterator itr = itens.iterator(); itr.hasNext();) {
            ItemPortfolio item = (ItemPortfolio) itr.next();

            geraComentarios(ve, item, tipo);

            List caminho = caminho(raiz, pasta);
            List comentarios = item.getComentarios();

            /* Ordenamos os comentarios de um item do portfolio em ordem decrescente de data */
            Collections.sort(comentarios, compData);

            context.put("comentarios", comentarios);
            context.put("pasta", pasta);
            context.put("todosGrupos", GrupoFactory.getInstance(curso));
            context.put("todosUsuarios", UsuarioFactory.getInstance(curso));
            context.put("portfolio", portfolio);
            context.put("tipo", tipo);
            context.put("caminho", caminho);
            context.put("raiz", raiz);
            context.put("usuarioAtual", usuarioAtual);
            context.put("itemPortfolio", item);

            Writer writer = createWriter(this.dirExtracao + "/itemPortfolio_"+ item.getId() +".html");
            try {
                t.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) { /* ignore */ }
            }
        }
    }

    /**
     * Gera comentarios dos itens do portfolio
     *
     * @param ve
     * @param item
     * @throws ResourceNotFoundException
     * @throws ParseErrorException
     * @throws Exception
     */
    private void geraComentarios(VelocityEngine ve, ItemPortfolio item, String tipo)
    throws ResourceNotFoundException, ParseErrorException, Exception {

        VelocityContext context = createVelocityContext();
        Template t = ve.getTemplate("templates/portfolio/comentarioPortfolio.vsl");

        List comentarios = item.getComentarios();
        LinkedList comentarios2 = new LinkedList();

        /* Neste momento estamos retirando os comentarios apagados e ordenando em ordem de data inversa */
        for (Iterator itr = comentarios.iterator(); itr.hasNext();) {
            Comentario coment = (Comentario) itr.next();
            if (!coment.getTexto().equals("")) {
                comentarios2.addFirst(coment);
            }
        }

        for (int i = 0; i < comentarios2.size(); i++) {
            Comentario comentario = (Comentario) comentarios2.get(i);

            Writer writer = createWriter(this.dirExtracao + "/comentarioPortfolio_"+ comentario.getId() +".html");

            Comentario proximoComentario = null;
            Comentario comentarioAnterior = null;

            if (i == comentarios2.size() - 1) {
                /* Ultimo comentario */
                proximoComentario = null;
            } else {
                proximoComentario = (Comentario) comentarios2.get(i + 1);
            }

            if (i > 0) {
                comentarioAnterior = (Comentario) comentarios2.get(i - 1);
            } else {
                /* Primeira mensagem */
                comentarioAnterior = null;
            }

            if (comentarioAnterior != null) {
                context.put("comentarioAnterior", comentarioAnterior);
            } else {
                context.put("comentarioAnterior", new Boolean(false));
            }
            if (proximoComentario != null) {
                context.put("proximoComentario", proximoComentario);
            } else {
                context.put("proximoComentario", new Boolean(false));
            }

            context.put("usuarioAtual", usuario);
            context.put("item", item);
            context.put("comentario", comentario);
            context.put("tipo", tipo);

            try {
                t.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) { /* ignore */ }
            }
        }
    }

    private void geraComponentesGrupo(VelocityEngine ve) throws ExtractionException, IOException, Exception {
        VelocityContext context = createVelocityContext();

        List grupos = Grupo.todosGrupos(curso);

        for (Iterator itr = grupos.iterator(); itr.hasNext();) {
            Grupo grupo = (Grupo) itr.next();
            List usuarios = grupo.getUsuarios();

            Writer writer = createWriter(this.dirExtracao + "/componentesGrupo_" + grupo.getId() + ".html");

            context.put("usuarios", usuarios);
            context.put("grupo", grupo);

            try {
                Template t = ve.getTemplate("templates/portfolio/componentesGrupo.vsl");
                t.merge(context, writer);
            } finally {
                try {
                    writer.flush();
                    writer.close();
                } catch (IOException ioex) { /* ignore */ }
            }
        }
    }

    /**
     * Gera o caminho da raiz at uma pasta especificada
     *
     * @param raiz
     * @param pasta
     * @return
     */
    private List caminho(PastaPortfolio raiz, PastaPortfolio pasta) {
        LinkedList caminho = new LinkedList();
        caminhoAux(raiz, pasta, caminho);
        return caminho;
    }

    /**
     * @param p1
     * @param p2
     * @param l
     * @return
     */
    private boolean caminhoAux(PastaPortfolio p1, PastaPortfolio p2, LinkedList l) {
        if (p1 == p2) {
            l.addFirst(p2);
            return true;
        } else {
            for (Iterator itr = p1.getSubPastas().iterator(); itr.hasNext();) {
                PastaPortfolio p = (PastaPortfolio) itr.next();
                if (caminhoAux(p, p2, l)) {
                    l.addFirst(p1);
                    return true;
                }
            }
            return false;
        }
    }
}
