/*
 * Created on Oct 15, 2004
 *
 */
package br.unicamp.nied.teleduc.extrator.usuario;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.database.DBException;

/**
 * @author leonel
 *
 * Factory de objetos Usurio. Para cada curso, existe um UsuarioFactory, que
 * funciona como um singleton. Faa:
 * <code>UsuarioFactory.getFactory(curso);</code>
 *
 */
public class UsuarioFactory {
    private Curso curso;
    private Map mapUsuarios = null;

    private static Map instances = new HashMap();

    /**
     * Constri uma Factory de usurios.
     *
     * @throws IOException
     * @throws DBException
     * @throws SQLException
     *
     */
    private UsuarioFactory(Curso curso) throws ExtractionException {
        this.curso = curso;
        this.mapUsuarios = new HashMap();
        preencher(curso);
    }

//    /**
//     * Constri um Factory de usurios, e popula a mesma com todos os usurios
//     * do curso, para otimizar a busca. Use no caso de precisar varrer todos os
//     * usurios
//     *
//     * @param curso
//     * @param preencher
//     * @throws SQLException
//     * @throws DBException
//     * @throws IOException
//     */
//    private UsuarioFactory(Curso curso, boolean preencher) throws ExtractionException {
//        super();
//        this.curso = curso;
//        this.mapUsuarios = new HashMap();
//        if (preencher) {
//            preencher(curso);
//        }
//    }

    /**
     * Retorna uma instncia de UsuarioFactory associada ao curso.
     *
     * @param Curso - o curso do qual retornar os usurios.
     */
    public static UsuarioFactory getInstance(Curso curso) throws ExtractionException {
        UsuarioFactory factory = (UsuarioFactory) instances.get(curso);
        if (factory == null) {
            factory = new UsuarioFactory(curso);
            instances.put(curso, factory);
        }
        return factory;
    }

    /**
     * Garante que o factory vai conter todos os usurios do curso.
     *
     * @param curso
     * @throws SQLException
     * @throws DBException
     * @throws IOException
     */
    private void preencher(Curso curso) throws ExtractionException {
        /*
         * Verifica todos os usurios do curso, para ver se esto no map.
         * Se o usurio no estiver no map, coloque-o l.
         */
        Collection todosUsuarios = Usuario.usuarios(curso);
        for (Iterator itr = todosUsuarios.iterator(); itr.hasNext();) {
            Usuario usr1 = (Usuario) itr.next();
            Usuario usr2 = this.getUsuario(usr1.getId());
            if (usr2 == null) {
                this.mapUsuarios.put(new Integer(usr1.getId()), usr1);
            }
        }
    }

    /**
     * Collection de objetos Usuario com todos os usurios do curso
     *
     * @return Collection com todos os usuarios do curso
     * @throws ExtractionException
     */
    public Collection todosUsuarios() throws ExtractionException {
        return this.mapUsuarios.values();
    }

    public Usuario getUsuario(int id) throws ExtractionException {
        return this.getUsuario(new Integer(id));
    }

    public Usuario getUsuario(Integer id) throws ExtractionException {
        /*if (id.intValue() < 0) {
            throw new IllegalArgumentException("O id do usurio no pode ser negativo");
        }*/
        Usuario usr = (Usuario) this.mapUsuarios.get(id);
        if (usr == null) {
            try {
                usr = new Usuario(this.curso, id.intValue());
                this.mapUsuarios.put(id, usr);
            } catch (DBException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } catch (SQLException e) {
                e.printStackTrace();
                return null;
            }
        }
        return usr;
    }
}
